@@ -20,6 +20,8 @@
 #undef GDA_PATCHLEVEL
 #undef HAVE_POSTGRES
 #undef HAVE_GTKSHEET
+#undef HAVE_SYBASE
+#undef HAVE_ORACLE
 /*
  * Backward compatibility
  */
