@@ -7,8 +7,8 @@ test -z "$srcdir" && srcdir=.
 PKG_NAME="Gnome Data Access"
 
 (test -f $srcdir/configure.in \
-  && test -d $srcdir/src \
-  && test -f $srcdir/src/gnome-dbadmin.c) || {
+  && test -d $srcdir/gda \
+  && test -f $srcdir/gda/gda.idl) || {
     echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
     echo " top-level gnome-filer directory"
     exit 1
