@@ -1,5 +1,6 @@
 /* GNOME DB libary
  * Copyright (C) 1998,1999 Michael Lausch
+ * Copyright (C) 1999 Rodrigo Moya
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,6 +20,7 @@
 #include <gnome.h>
 #include <gda.h>
 #include <libgnorba/gnorba.h>
+#include <gtk/gtkmarshal.h>
 
 #include "gda-connection.h"
 #include "gda-server.h"
@@ -27,15 +29,197 @@
 
 static GHashTable* factories = 0;
 
+enum
+{
+  CONNECTION_ERROR,
+  CONNECTION_WARNING,
+  CONNECTION_OPEN,
+  CONNECTION_CLOSE,
+  LAST_SIGNAL
+};
 
+static gint
+gda_connection_signals[LAST_SIGNAL] = {0, };
+
+static void    gda_connection_class_init    (Gda_ConnectionClass* klass);
+static void    gda_connection_init          (Gda_Connection* cnc);
+static void    gda_connection_real_error    (Gda_Connection* cnc, GList*);
+static void    gda_connection_real_warning  (Gda_Connection* cnc, GList*);
+static void    gda_connection_real_open     (Gda_Connection* cnc);
+static void    gda_connection_real_close    (Gda_Connection* cnc);
+static int     get_corba_connection         (Gda_Connection* cnc);
+
+static void
+gda_connection_real_error (Gda_Connection* cnc, GList* errors)
+{
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+  cnc->errors_head = g_list_concat(cnc->errors_head, errors);
+}
+
+static void
+gda_connection_real_warning (Gda_Connection* cnc, GList* warnings)
+{
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+}
+
+/**
+ * gda_connection_real_open:
+ * @cnc: the connection object which describes the server and
+ *        the database name to which the connection shuld be opened
+ * @dsn: The DSN of the database. Can be NULL.
+ * @user: The username for authentication to the database. Can be NULL.
+ * @pwd: The password for authentication to the database. Can be NULL.
+ * 
+ * The function activates the correct CORBA server (defined with he
+ * #gda_connection_set_provider() function. Then it
+ * tries to open the database using the DSN or the default database
+ * as a data source. If @user or @pwd is not NULL, it will overwrite the
+ * appropriate entry in the DSN passed as @par2. Entries in the DSN have the form
+ * <key> = <value> seperated from the database name . Currently the DSN is not
+ * parsed.
+ *
+ * Returns: 0 on success, -1 on error
+ */
+static void
+gda_connection_real_open (Gda_Connection* cnc)
+{
+  CORBA_Environment     ev;
+  gint                  rc;
+  
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+  
+  if (get_corba_connection(cnc) < 0)
+      return;
+
+  CORBA_exception_init(&ev);
+
+  rc = GDA_Connection_open(cnc->connection, cnc->database, cnc->user, cnc->passwd, &ev);
+  g_print("GDA_Connection_open returns %d\n", rc);
+  if (gda_connection_corba_exception(cnc, &ev) < 0)
+    {
+      CORBA_SystemException* sysexc = CORBA_exception_value(&ev);
+      if (sysexc->completed != CORBA_COMPLETED_NO)
+	{
+	  GDA_Connection_close(cnc->connection, &ev);
+	  CORBA_Object_release(cnc->connection, &ev);
+	}
+      cnc->connection = CORBA_OBJECT_NIL;
+      return ;
+    }
+  
+  cnc->is_open = 1;
+  return;
+}
+
+/**
+ * gda_connection_real_close:
+ * @cnc: The connection object which should be closed
+ *
+ * Closes the connection object and all associated #Gda_Recordset objects.
+ * #Gda_Command objects are not closed, but can't be used anymore. Transactions
+ * pending on this objects are aborted and an error is returned.
+ *
+ * Returns: 0 on success, -1 on error
+ */
+static void
+gda_connection_real_close (Gda_Connection* cnc)
+{
+  CORBA_Environment     ev;
+  
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+
+  g_return_if_fail(cnc != NULL);
+  g_return_if_fail(cnc->connection != NULL);
+
+  CORBA_exception_init(&ev);
+  GDA_Connection_close(cnc->connection, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+  CORBA_Object_release(cnc->connection, &ev);
+  cnc->is_open = 0;
+  cnc->connection = CORBA_OBJECT_NIL;
+  return;
+}
+
+guint
+gda_connection_get_type (void)
+{
+  static guint gda_connection_type = 0;
+
+  if (!gda_connection_type)
+    {
+      GtkTypeInfo gda_connection_info =
+      {
+	"GdaConnection",
+	sizeof (Gda_Connection),
+	sizeof (Gda_ConnectionClass),
+	(GtkClassInitFunc)gda_connection_class_init,
+	(GtkObjectInitFunc)gda_connection_init,
+	(GtkArgSetFunc)NULL,
+	(GtkArgSetFunc)NULL,
+      };
+      gda_connection_type = gtk_type_unique(gtk_object_get_type(),
+                                            &gda_connection_info);
+    }
+  return gda_connection_type;
+}
+
+
+static void
+gda_connection_class_init (Gda_ConnectionClass* klass)
+{
+  GtkObjectClass*   object_class;
+
+  object_class = (GtkObjectClass*) klass;
+  
+  gda_connection_signals[CONNECTION_ERROR] = gtk_signal_new("error",
+							    GTK_RUN_FIRST,
+							    object_class->type,
+							    GTK_SIGNAL_OFFSET(Gda_ConnectionClass, error),
+							    gtk_marshal_NONE__POINTER,
+							    GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+  gda_connection_signals[CONNECTION_WARNING] = gtk_signal_new("warning",
+							    GTK_RUN_LAST,
+							    object_class->type,
+							    GTK_SIGNAL_OFFSET(Gda_ConnectionClass, warning),
+							    gtk_marshal_NONE__POINTER,
+							    GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+  gda_connection_signals[CONNECTION_OPEN] = gtk_signal_new("open",
+							    GTK_RUN_LAST,
+							    object_class->type,
+							    GTK_SIGNAL_OFFSET(Gda_ConnectionClass, open),
+							    gtk_signal_default_marshaller,
+							    GTK_TYPE_NONE, 0);
+  gda_connection_signals[CONNECTION_CLOSE] = gtk_signal_new("close",
+							    GTK_RUN_LAST,
+							    object_class->type,
+							    GTK_SIGNAL_OFFSET(Gda_ConnectionClass, close),
+							    gtk_signal_default_marshaller,
+							    GTK_TYPE_NONE, 0);
+
+  gtk_object_class_add_signals(object_class, gda_connection_signals, LAST_SIGNAL);
+  klass->error   = gda_connection_real_error;
+  klass->warning = gda_connection_real_warning;
+  klass->open    = gda_connection_real_open;
+  klass->close   = gda_connection_real_close;
+  
+}
+
+static void
+gda_connection_init (Gda_Connection* cnc)
+{
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+}
+
+
+  
 static guint
-hash_id_hash(gconstpointer data)
+hash_id_hash (gconstpointer data)
 {
   return (guint)data;
 }
 
 static gint
-guint_id_guint(gconstpointer d1, gconstpointer d2)
+guint_id_guint (gconstpointer d1, gconstpointer d2)
 {
   return d1 < d2 ? -1 : d1 == d2 ? 0 : 1;
 }
@@ -45,7 +229,7 @@ guint_id_guint(gconstpointer d1, gconstpointer d2)
  * error objects are here
  */
 static gint
-Exception(CORBA_Environment *ev)
+Exception (CORBA_Environment *ev)
 {
   switch (ev->_major)
     {
@@ -60,8 +244,79 @@ Exception(CORBA_Environment *ev)
   return 0;
 }
 
+gint
+gda_connection_corba_exception (Gda_Connection* cnc, CORBA_Environment* ev)
+{
+  Gda_Error* error;
+  
+  g_return_val_if_fail(ev != 0, -1);
+  
+  switch (ev->_major)
+    {
+    case CORBA_NO_EXCEPTION:
+      return 0;
+    case CORBA_SYSTEM_EXCEPTION:
+      {
+	CORBA_SystemException* sysexc;
+
+	sysexc = CORBA_exception_value(ev);
+	error = gda_error_new();
+	error->source = g_strdup("[CORBA System Exception]");
+	switch(sysexc->minor)
+	  {
+	  case ex_CORBA_COMM_FAILURE:
+	    error->description = g_strdup_printf(_("%s: The server didn't respond."), CORBA_exception_id(ev));
+	    break;
+	  default:
+	    error->description = g_strdup(_("%s: An Error occured in the CORBA system."));
+	    break;
+	  }
+	gda_connection_add_single_error(cnc, error);
+	return -1;
+      }
+    case CORBA_USER_EXCEPTION:
+      if (strcmp(CORBA_exception_id(ev), ex_GDA_NotSupported) == 0)
+	{
+	  GDA_NotSupported* not_supported_error;
+
+	  not_supported_error = (GDA_NotSupported*)ev->_params;
+	  error = gda_error_new();
+	  error->source = g_strdup("[CORBA User Exception]");
+	  error->description = g_strdup(not_supported_error->errormsg);
+	  gda_connection_add_single_error(cnc, error);
+	  return -1;
+	}
+      if (strcmp(CORBA_exception_id(ev), ex_GDA_DriverError) == 0)
+	{
+	  GDA_ErrorSeq  error_sequence = ((GDA_DriverError*)ev->_params)->errors;
+	  gint          idx;
+
+	  for (idx = 0; idx < error_sequence._length; idx++)
+	    {
+	      GDA_Error* gda_error = &error_sequence._buffer[idx];
+
+	      error = gda_error_new();
+	      if (gda_error->source)
+		error->source = g_strdup(gda_error->source);
+	      error->number = gda_error->number;
+	      if (gda_error->sqlstate)
+		error->sqlstate = g_strdup(gda_error->sqlstate);
+	      if (gda_error->nativeMsg)
+		error->nativeMsg = g_strdup(gda_error->nativeMsg);
+	      if (gda_error->description)
+		error->description = g_strdup(gda_error->description);
+	      gda_connection_add_single_error(cnc, error);
+	    }
+	}
+      return -1;
+    default:
+      g_error("Unknown CORBA exception for connection");
+    }
+  return 0;
+}
+
 static int
-get_corba_connection(Gda_Connection* cnc)
+get_corba_connection (Gda_Connection* cnc)
 {
   CORBA_Environment ev;
   GDA_ConnectionFactory factory;
@@ -80,31 +335,29 @@ get_corba_connection(Gda_Connection* cnc)
       fprintf(stderr,"Cannot open factory\n");
       return -1;
     }
-  cnc->connection = GDA_ConnectionFactory_create_connection(factory, "gda-odbc", &ev);
+  cnc->connection = GDA_ConnectionFactory_create_connection(factory, cnc->provider, &ev);
   if (Exception(&ev) < 0)
     return -1;
   return 0;
 }
 
-
-
 /**
  * gda_connection_new:
- * @orb: The ORB
+ * @orb: the ORB
  *
  * Allocates space for a client side connection object
  *
  * Returns: the pointer to the allocated object
  */
-
 Gda_Connection*
-gda_connection_new(CORBA_ORB orb)
+gda_connection_new (CORBA_ORB orb)
 {
   Gda_Connection* cnc;
   
-  g_return_val_if_fail(orb != NULL, NULL);
-  
-  cnc = g_new0(Gda_Connection, 1);
+  g_return_val_if_fail(orb != 0, 0);
+
+  cnc = gtk_type_new(gda_connection_get_type());
+  cnc->orb = orb;
   return cnc;
 }
 
@@ -118,15 +371,17 @@ gda_connection_new(CORBA_ORB orb)
  *
  */
 void
-gda_connection_free(Gda_Connection* cnc)
+gda_connection_free (Gda_Connection* cnc)
 {
-  if (cnc->is_open)
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  
+  if (gda_connection_is_open(cnc))
     {
       if (cnc->commands)
 	g_warning("Commands still associated with gda_connection");
       if (cnc->recordsets)
 	g_warning("Recordsets still associated with gda_connection");
-      if (cnc->errors)
+      if (cnc->errors_head)
 	g_warning("Errors still associated with gda_connection");
       gda_connection_close(cnc);
     }
@@ -143,7 +398,6 @@ gda_connection_free(Gda_Connection* cnc)
   g_free(cnc);
 }  
 
-
 /**
  * gda_connection_set_provider:
  * @cnc: connection object
@@ -154,11 +408,10 @@ gda_connection_free(Gda_Connection* cnc)
  * use
  *
  */
-
 void
-gda_connection_set_provider(Gda_Connection* cnc, gchar* name)
+gda_connection_set_provider (Gda_Connection* cnc, gchar* name)
 {
-  g_return_if_fail(cnc != 0);
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
   g_return_if_fail(name != 0);
 
   if (cnc->provider)
@@ -176,11 +429,11 @@ gda_connection_set_provider(Gda_Connection* cnc, gchar* name)
  */
  
 const gchar*
-gda_connection_get_provider(Gda_Connection* cnc)
+gda_connection_get_provider (Gda_Connection* cnc)
 {
-  g_return_val_if_fail(cnc != NULL, NULL);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
 
-  return g_strdup(cnc->provider);
+  return (g_strdup(cnc->provider));
 }
 
 /**
@@ -192,13 +445,15 @@ gda_connection_get_provider(Gda_Connection* cnc)
  * Returns: a GList with the DSNs of all known data sources.
  */
 GList*
-gda_connection_list_datasources(Gda_Connection* cnc)
+gda_connection_list_datasources (Gda_Connection* cnc)
 {
   CORBA_Environment       ev;
   GDA_Connection_DSNlist* dsnlist;
   GList*                  retval = 0;
   gint                    idx;
   
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  
   CORBA_exception_init(&ev);
   if (!cnc->connection)
     get_corba_connection(cnc);
@@ -226,10 +481,10 @@ gda_connection_list_datasources(Gda_Connection* cnc)
  *
  */
 void
-gda_connection_set_default_db(Gda_Connection* cnc, gchar* dsn)
+gda_connection_set_default_db (Gda_Connection* cnc, gchar* dsn)
 {
-  g_return_if_fail(cnc != NULL);
-  g_return_if_fail(dsn != NULL);
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  g_return_if_fail(dsn != 0);
 
   if (cnc->default_db)
     g_free(cnc->default_db);
@@ -254,66 +509,57 @@ gda_connection_set_default_db(Gda_Connection* cnc, gchar* dsn)
  *
  * Returns: 0 on success, -1 on error
  */
+
 gint
-gda_connection_open(Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
+gda_connection_open (Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
 {
-  gchar*                db_to_use;
-  gchar*                user_to_use;
-  gchar*                pwd_to_use;
-  CORBA_Environment     ev;
-  gint                  rc;
+  gchar* db_to_use;
+  gchar* user_to_use;
+  gchar* pwd_to_use;
   
-  g_return_val_if_fail(cnc != NULL, -1);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
   g_return_val_if_fail(cnc->connection == CORBA_OBJECT_NIL, -1);
-  g_return_val_if_fail(user != NULL, -1);
-  g_return_val_if_fail(pwd != NULL, -1);
+  g_return_val_if_fail(user != 0, -1);
+  g_return_val_if_fail(pwd != 0, -1);
   g_return_val_if_fail(cnc->is_open == 0, -1);
-  
+
   if (!dsn)
     db_to_use = cnc->default_db;
   else
     {
-      size_t pos = strcspn(dsn, ";");
+      size_t pos = strcspn(dsn, "=");
       if (pos != strlen(dsn))
-	db_to_use = g_strdup(dsn);
+	{
+	  db_to_use = g_strdup(dsn);
+	}
       else
 	{
 	  gchar path[128];
 	  g_snprintf(path, sizeof(path), "/gdalib/%s/DSN", dsn);
 	  db_to_use = gnome_config_get_string(path);
 	}
+      if (!db_to_use)
+	{
+	  Gda_Error* e = gda_error_new();
+	  
+	  e->description = g_strdup_printf(_("Database '%s' not found in system configuration"), dsn);
+	  e->source = g_strdup("[GDA Client Library]");
+	  e->nativeMsg = g_strdup(e->description);
+	  gda_connection_add_single_error(cnc, e);
+	  return -1;
+	}
     }
+      			 
   user_to_use = user;
   pwd_to_use = pwd;
 
-  if (!db_to_use)
-    {
-      Gda_Error* e = gda_error_new();
-
-      e->description = g_strdup(_("Database '%s' not found in system configuration"));
-      e->source = g_strdup("GDA Client Library");
-      e->native = g_strdup(e->description);
-      cnc->errors = g_list_append(cnc->errors, e);
-      return -1;
-    }
-      			 
   cnc->database = g_strdup(db_to_use);
   cnc->user = g_strdup(user_to_use);
   cnc->passwd = g_strdup(pwd_to_use);
 
-  if (get_corba_connection(cnc) < 0)
-      return -1;
-
-  rc = GDA_Connection_open(cnc->connection, db_to_use, user_to_use, pwd_to_use, &ev);
-  if (Exception(&ev) < 0)
-    return -1;
-  
-  if (rc < 0)
-    {
-      fprintf(stderr,"connection opend failed\n");
-      return -1;
-    }
-  cnc->is_open = 1;
+  gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_OPEN]);
+  if (!cnc->is_open)
+    return -1 ;
   return 0;
 }
 
@@ -327,34 +573,16 @@ gda_connection_open(Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
  *
  * Returns: 0 on success, -1 on error
  */
-gint
-gda_connection_close(Gda_Connection* cnc)
+void
+gda_connection_close (Gda_Connection* cnc)
 {
-  CORBA_Environment     ev;
-  gint                  rc = 0;
-  
-  g_return_val_if_fail(cnc != NULL, -1);
-  g_return_val_if_fail(cnc->connection != NULL, -1);
-
-  /* Here we should notify all recordset and command objects */
-  
-  CORBA_exception_init(&ev);
-  GDA_Connection_close(cnc->connection, &ev);
-  if (Exception(&ev) < 0)
-    rc = -1;
-  cnc->is_open = 0;
-  CORBA_Object_release(cnc->connection, &ev);
-  cnc->connection = CORBA_OBJECT_NIL;
-  Exception(&ev);
-  return rc;
+  gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_CLOSE]);
 }
 
 
-
-
 static Gda_Recordset*
-open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
-	    GDA_Connection_ConstraintSeq* constraints)
+open_schema (Gda_Connection* cnc, GDA_Connection_QType t,
+	     GDA_Connection_ConstraintSeq* constraints)
 {
   CORBA_Environment ev;
   CORBA_Object      corba_rs;
@@ -365,7 +593,7 @@ open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
   corba_rs = GDA_Connection_openSchema(cnc->connection, t,constraints, &ev);
   if (Exception(&ev) < 0 || corba_rs == CORBA_OBJECT_NIL)
     return 0;
-  rs = gda_recordset_new();
+  rs = GDA_RECORDSET(gda_recordset_new());
   rs->open = 1;
   rs->corba_rs = corba_rs;
   rs->field_attributes = GDA_Recordset_describe(corba_rs, &ev);
@@ -373,23 +601,6 @@ open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
     {
       fprintf(stderr," Cannot get recordset description\n");
     }
-#if 0
-  {
-    gint idx;
-    
-    for (idx = 0; idx  < rs->field_attributes->_length; idx++)
-      {
-	gchar type_bfr[120];
-	fprintf(stderr,"Description for field '%s'\n", rs->field_attributes->_buffer[idx].name);
-	fprintf(stderr,"     defined Size[prec] = %d\n", rs->field_attributes->_buffer[idx].definedSize);
-	fprintf(stderr,"     scale              = %d\n", rs->field_attributes->_buffer[idx].scale);
-	fprintf(stderr,"     gdaType            = '%s'\n", gda_gdatype_2_string(type_bfr, sizeof(type_bfr),
-									    rs->field_attributes->_buffer[idx].gdaType));
-	fprintf(stderr,"     cType              = %d\n", rs->field_attributes->_buffer[idx].cType);
-	fprintf(stderr,"     nativeType         = %d\n", rs->field_attributes->_buffer[idx].nativeType);
-      }
-  }
-#endif
   return rs;
 }
 
@@ -409,7 +620,7 @@ open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
  */
 
 Gda_Recordset*
-gda_connection_open_schema_array(Gda_Connection* cnc, GDA_Connection_QType t, Gda_Constraint_Element* elems)
+gda_connection_open_schema_array (Gda_Connection* cnc, GDA_Connection_QType t, Gda_Constraint_Element* elems)
 {
 
   CORBA_Environment ev;
@@ -418,7 +629,7 @@ gda_connection_open_schema_array(Gda_Connection* cnc, GDA_Connection_QType t, Gd
   gint                          index;
   Gda_Constraint_Element*       current;
   
-  g_return_val_if_fail(cnc != 0, 0);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
   g_return_val_if_fail(cnc->connection != 0, 0);
 
   count = 0;
@@ -467,9 +678,8 @@ gda_connection_open_schema_array(Gda_Connection* cnc, GDA_Connection_QType t, Gd
  */
 
 Gda_Recordset*
-gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
+gda_connection_open_schema (Gda_Connection* cnc, GDA_Connection_QType t, ...)
 {
-
   va_list ap;
   CORBA_Environment ev;
   GDA_Connection_ConstraintSeq* constraints;
@@ -481,7 +691,8 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
   gint                          index;
   Gda_Recordset*                rc;
   
-  g_return_val_if_fail(cnc != 0, 0);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  g_return_val_if_fail(gda_connection_is_open(cnc), 0);
   g_return_val_if_fail(cnc->connection != 0, 0);
 
   va_start(ap, t);
@@ -543,14 +754,14 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
  * Returns: a #Glist holding the error objects.
  */
 GList*
-gda_connection_get_errors(Gda_Connection* cnc)
+gda_connection_get_errors (Gda_Connection* cnc)
 {
   GList*             retval;
   GDA_ErrorSeq*      remote_errors;
   CORBA_Environment  ev;
   gint               idx;
 
-  g_return_val_if_fail(cnc != NULL, 0);  
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
   CORBA_exception_init(&ev);
   
   if (cnc->connection)
@@ -566,14 +777,14 @@ gda_connection_get_errors(Gda_Connection* cnc)
 	  e->source      = g_strdup(remote_errors->_buffer[idx].source);
 	  e->helpurl = 0;
 	  e->sqlstate    = g_strdup(remote_errors->_buffer[idx].sqlstate);
-	  e->native      = g_strdup(remote_errors->_buffer[idx].nativeMsg);
-	  cnc->errors = g_list_append(cnc->errors, e);
+	  e->nativeMsg   = g_strdup(remote_errors->_buffer[idx].nativeMsg);
+	  gda_connection_add_single_error(cnc, e);
 	}
       CORBA_free(remote_errors);
     }
   
-  retval = cnc->errors;
-  cnc->errors = 0;
+  retval = cnc->errors_head;
+  cnc->errors_head = 0;
   return retval;
 }
 
@@ -586,10 +797,13 @@ gda_connection_get_errors(Gda_Connection* cnc)
  * Returns: 0 if everything is okay, -1 on error
  */
 gint
-gda_connection_begin_transaction(Gda_Connection* cnc)
+gda_connection_begin_transaction (Gda_Connection* cnc)
 {
   CORBA_Environment ev;
 
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  g_return_val_if_fail(gda_connection_is_open(cnc), -1);
+  
   CORBA_exception_init(&ev);
   
   GDA_Connection_beginTransaction(cnc->connection, &ev);
@@ -598,7 +812,6 @@ gda_connection_begin_transaction(Gda_Connection* cnc)
   return 0;
 }
 
-
 /**
  * gda_connection_commit_transaction:
  * @cnc: the connection object
@@ -608,21 +821,23 @@ gda_connection_begin_transaction(Gda_Connection* cnc)
  * Returns: 0 if everything is okay, -1 on error
  */
 gint
-gda_connection_commit_transaction(Gda_Connection* cnc)
+gda_connection_commit_transaction (Gda_Connection* cnc)
 {
   CORBA_Environment ev;
 
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  g_return_val_if_fail(gda_connection_is_open(cnc), -1);
+  
   CORBA_exception_init(&ev);
   
   GDA_Connection_commitTransaction(cnc->connection, &ev);
   if (Exception(&ev) < 0)
     return -1;
   return 0;
-  return 0;
 }
 
 /**
- * gda_connection_abort_transaction:
+ * gda_connection_rollback_transaction:
  * @cnc: the connection object
  *
  * Abort and rollback a pending transaction on @cnc
@@ -631,17 +846,19 @@ gda_connection_commit_transaction(Gda_Connection* cnc)
  */
 
 gint
-gda_connection_rollback_transaction(Gda_Connection* cnc)
+gda_connection_rollback_transaction (Gda_Connection* cnc)
 {
   CORBA_Environment ev;
 
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  g_return_val_if_fail(gda_connection_is_open(cnc), -1);
+  
   CORBA_exception_init(&ev);
   
   GDA_Connection_rollbackTransaction(cnc->connection, &ev);
   if (Exception(&ev) < 0)
     return -1;
   return 0;
-  return 0;
 }
 
 
@@ -660,11 +877,15 @@ gda_connection_rollback_transaction(Gda_Connection* cnc)
  * Returns: a recordset on success or NULL on error.
  */
 Gda_Recordset*
-gda_connection_execute(Gda_Connection* cnc, gchar* txt, gulong* reccount, gulong flags)
+gda_connection_execute (Gda_Connection* cnc, gchar* txt, gulong* reccount, gulong flags)
 {
   Gda_Command*   cmd;
   Gda_Recordset* rs;
   
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  g_return_val_if_fail(gda_connection_is_open(cnc), 0);
+  g_return_val_if_fail(txt != 0, 0);
+  
   cmd = gda_command_new();
   gda_command_set_connection(cmd, cnc);
   gda_command_set_text(cmd, txt);
@@ -684,14 +905,15 @@ gda_connection_execute(Gda_Connection* cnc, gchar* txt, gulong* reccount, gulong
  * Returns: -1 on error and 0 if everything is okay
  */
 gint
-gda_connection_start_logging(Gda_Connection* cnc, gchar* filename)
+gda_connection_start_logging (Gda_Connection* cnc, gchar* filename)
 {
   CORBA_Environment ev;
   gint              rc;
   
-  g_return_val_if_fail(cnc, -1);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  g_return_val_if_fail(gda_connection_is_open(cnc), -1);
   g_return_val_if_fail(filename, -1);
-  g_return_val_if_fail(cnc->connection, -1);
+  g_return_val_if_fail(cnc->connection != 0, -1);
 
   CORBA_exception_init(&ev);
   rc = GDA_Connection_startLogging(cnc->connection, filename, &ev);
@@ -703,23 +925,22 @@ gda_connection_start_logging(Gda_Connection* cnc, gchar* filename)
 }
 
 /**
- * gda_connection_start_logging:
+ * gda_connection_stop_logging:
  * @cnc: the connection
- * @filename: the name of the logfile
  *
- * This function causes the server to log the statements
- * it executes in @filename.
+ * This function stops the database server logs
  *
  * Returns: -1 on error and 0 if everything is okay
  */
 gint
-gda_connection_stop_logging(Gda_Connection* cnc)
+gda_connection_stop_logging (Gda_Connection* cnc)
 {
   CORBA_Environment ev;
   gint              rc;
   
-  g_return_val_if_fail(cnc, -1);
-  g_return_val_if_fail(cnc->connection, -1);
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  g_return_val_if_fail(gda_connection_is_open(cnc), -1);
+  g_return_val_if_fail(cnc->connection != 0, -1);
 
   CORBA_exception_init(&ev);
   rc = GDA_Connection_stopLogging(cnc->connection, &ev);
@@ -732,12 +953,16 @@ gda_connection_stop_logging(Gda_Connection* cnc)
   
     
 gchar*
-gda_connection_create_recordset(Gda_Connection* cnc, Gda_Recordset* rs)
+gda_connection_create_recordset (Gda_Connection* cnc, Gda_Recordset* rs)
 {
 
   gchar*            retval;
   CORBA_Environment ev;
 
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  g_return_val_if_fail(gda_connection_is_open(cnc), 0);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
+  
   CORBA_exception_init(&ev);
   retval = GDA_Connection_createTable(cnc->connection,
 				      rs->name,
@@ -746,3 +971,136 @@ gda_connection_create_recordset(Gda_Connection* cnc, Gda_Recordset* rs)
     return 0;
   return retval;
 }
+
+void
+gda_connection_add_single_error (Gda_Connection* cnc, Gda_Error* error)
+{
+  GList* error_list = 0;
+  
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  g_return_if_fail(error != 0);
+
+  error_list = g_list_append(error_list, error);
+  gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_ERROR], error_list);
+}
+
+void
+gda_connection_add_errorlist (Gda_Connection* cnc, GList* errors)
+{
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  g_return_if_fail(errors != 0);
+  
+  gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_ERROR], errors);
+}
+
+glong
+gda_connection_get_flags (Gda_Connection *cnc)
+{
+  CORBA_Environment ev;
+  glong ret;
+  
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  
+  CORBA_exception_init(&ev);
+  ret = GDA_Connection__get_flags(cnc->connection, &ev);
+  if (gda_connection_corba_exception(cnc, &ev) == 0) return (ret);
+  return (-1);
+}
+
+void
+gda_connection_set_flags (Gda_Connection *cnc, glong flags)
+{
+  CORBA_Environment ev;
+  
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  
+  CORBA_exception_init(&ev);
+  GDA_Connection__set_flags(cnc->connection, (CORBA_long) flags, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+}
+
+glong
+gda_connection_get_cmd_timeout (Gda_Connection *cnc)
+{
+  CORBA_Environment ev;
+  glong ret;
+  
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  
+  CORBA_exception_init(&ev);
+  ret = GDA_Connection__get_cmdTimeout(cnc->connection, &ev);
+  if (gda_connection_corba_exception(cnc, &ev) == 0) return (ret);
+  return (-1);
+}
+
+void
+gda_connection_set_cmd_timeout (Gda_Connection *cnc, glong cmd_timeout)
+{
+  CORBA_Environment ev;
+  
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  
+  CORBA_exception_init(&ev);
+  GDA_Connection__set_cmdTimeout(cnc->connection, (CORBA_long) cmd_timeout, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+}
+
+glong
+gda_connection_get_connect_timeout (Gda_Connection *cnc)
+{
+  CORBA_Environment ev;
+  glong ret;
+  
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  
+  CORBA_exception_init(&ev);
+  ret = GDA_Connection__get_connectTimeout(cnc->connection, &ev);
+  if (gda_connection_corba_exception(cnc, &ev) == 0) return (ret);
+  return (-1);
+}
+
+void
+gda_connection_set_connect_timeout (Gda_Connection *cnc, glong timeout)
+{
+  CORBA_Environment ev;
+  
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  
+  CORBA_exception_init(&ev);
+  GDA_Connection__set_connectTimeout(cnc->connection, (CORBA_long) timeout, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+}
+
+GDA_CursorLocation
+gda_connection_get_cursor_location (Gda_Connection *cnc)
+{
+  CORBA_Environment ev;
+  GDA_CursorLocation cursor;
+  
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
+  
+  CORBA_exception_init(&ev);
+  cursor = GDA_Connection__get_cursor(cnc->connection, &ev);
+  if (gda_connection_corba_exception(cnc, &ev) == 0) return (cursor);
+  return (-1);
+}
+
+void
+gda_connection_set_cursor_location (Gda_Connection *cnc, GDA_CursorLocation cursor)
+{
+  CORBA_Environment ev;
+  
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+  
+  CORBA_exception_init(&ev);
+  GDA_Connection__set_cursor(cnc->connection, cursor, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+}
+
+gchar *
+gda_connection_get_version (Gda_Connection *cnc)
+{
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  g_error("%s: not implemented", __PRETTY_FUNCTION__);
+  return (0);
+}
\ No newline at end of file
