@@ -1,5 +1,6 @@
 /* GNOME DB libary
  * Copyright (C) 1998,1999 Michael Lausch
+ * Copyright (C) 1999 Rodrigo Moya
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,32 +22,73 @@
 
 #include "gda-field.h"
 
+enum
+{
+  GDA_FIELD_LAST_SIGNAL
+};
 
+static guint gda_field_signals[GDA_FIELD_LAST_SIGNAL] = { 0, };
+
+static void gda_field_class_init (Gda_FieldClass *klass);
+static void gda_field_init       (Gda_Field *field);
 
 #define ENUM_TO_STR(e) case (e): strncpy(bfr, #e, length); break
 #define min(a,b)       (a) < (b) ? (a) : (b)
 
+guint
+gda_field_get_type (void)
+{
+  static guint gda_field_type = 0;
 
+  if (!gda_field_type)
+    {
+      GtkTypeInfo gda_field_info =
+      {
+	"GdaField",
+	sizeof (Gda_Field),
+	sizeof (Gda_FieldClass),
+	(GtkClassInitFunc) gda_field_class_init,
+	(GtkObjectInitFunc) gda_field_init,
+	(GtkArgSetFunc) NULL,
+	(GtkArgSetFunc) NULL,
+      };
+      gda_field_type = gtk_type_unique(gtk_object_get_type(), &gda_field_info);
+    }
+  return gda_field_type;
+}
+
+
+static void
+gda_field_class_init (Gda_FieldClass* klass)
+{
+  GtkObjectClass*   object_class;
+
+  object_class = (GtkObjectClass*) klass;  
+}
+
+static void
+gda_field_init (Gda_Field* field)
+{
+}
 
 /**
  * gda_fieldtype_2_string:
  * @bfr: bufferspace where the printed name of the type of the field
  * is placed into.
  * @length: the size of the buffer
- * @f: a pointer to the field.
  *
  * Returns: a pointer to the buffer.
  */
 
 gchar* 
-gda_fieldtype_2_string(gchar* bfr, gint length, Gda_Field* f)
+gda_fieldtype_2_string(gchar* bfr, gint length, GDA_ValueType type)
 {
   if (!bfr)
     {
       bfr = g_new0(gchar, 20);
       length = 20;
     }
-  switch (f->attributes->gdaType)
+  switch (type)
     {
       ENUM_TO_STR(GDA_TypeNull);
       ENUM_TO_STR(GDA_TypeBigint);
@@ -82,12 +124,59 @@ gda_fieldtype_2_string(gchar* bfr, gint length, Gda_Field* f)
   return bfr;
 }
 
+/**
+ * gda_string_2_fieldtype
+ * @type GDA type string
+ *
+ * Returns the GDA type identifier for the given string (returned by
+ * gda_fieldtype_2_string
+ */
+GDA_ValueType
+gda_string_2_fieldtype (gchar *type)
+{
+  g_return_val_if_fail(type != 0, -1);
+  
+  /* FIXME: is there a better way of doing this? */
+  if (!strcmp(type, "GDA_TypeNull")) return (GDA_TypeNull);
+  else if (!strcmp(type, "GDA_TypeBigint")) return (GDA_TypeBigint);
+  else if (!strcmp(type, "GDA_TypeBinary")) return (GDA_TypeBinary);
+  else if (!strcmp(type, "GDA_TypeBoolean")) return (GDA_TypeBoolean);
+  else if (!strcmp(type, "GDA_TypeBstr")) return (GDA_TypeBstr);
+  else if (!strcmp(type, "GDA_TypeChar")) return (GDA_TypeChar);
+  else if (!strcmp(type, "GDA_TypeCurrency")) return (GDA_TypeCurrency);
+  else if (!strcmp(type, "GDA_TypeDate")) return (GDA_TypeDate);
+  else if (!strcmp(type, "GDA_TypeDbDate")) return (GDA_TypeDbDate);
+  else if (!strcmp(type, "GDA_TypeDbTime")) return (GDA_TypeDbTime);
+  else if (!strcmp(type, "GDA_TypeDbTimestamp")) return (GDA_TypeDbTimestamp);
+  else if (!strcmp(type, "GDA_TypeDecimal")) return (GDA_TypeDecimal);
+  else if (!strcmp(type, "GDA_TypeDouble")) return (GDA_TypeDouble);
+  else if (!strcmp(type, "GDA_TypeError")) return (GDA_TypeError);
+  else if (!strcmp(type, "GDA_TypeInteger")) return (GDA_TypeInteger);
+  else if (!strcmp(type, "GDA_TypeLongvarbin")) return (GDA_TypeLongvarbin);
+  else if (!strcmp(type, "GDA_TypeLongvarchar")) return (GDA_TypeLongvarchar);
+  else if (!strcmp(type, "GDA_TypeLongvarwchar")) return (GDA_TypeLongvarwchar);
+  else if (!strcmp(type, "GDA_TypeNumeric")) return (GDA_TypeNumeric);
+  else if (!strcmp(type, "GDA_TypeSingle")) return (GDA_TypeSingle);
+  else if (!strcmp(type, "GDA_TypeSmallint")) return (GDA_TypeSmallint);
+  else if (!strcmp(type, "GDA_TypeTinyint")) return (GDA_TypeTinyint);
+  else if (!strcmp(type, "GDA_TypeUBigint")) return (GDA_TypeUBigint);
+  else if (!strcmp(type, "GDA_TypeUSmallint")) return (GDA_TypeUSmallint);
+  else if (!strcmp(type, "GDA_TypeVarchar")) return (GDA_TypeVarchar);
+  else if (!strcmp(type, "GDA_TypeVarbin")) return (GDA_TypeVarbin);
+  else if (!strcmp(type, "GDA_TypeVarwchar")) return (GDA_TypeVarwchar);
+  else if (!strcmp(type, "GDA_TypeFixchar")) return (GDA_TypeFixchar);
+  else if (!strcmp(type, "GDA_TypeFixbin")) return (GDA_TypeFixbin);
+  else if (!strcmp(type, "GDA_TypeFixwchar")) return (GDA_TypeFixwchar);
+  return (-1);
+}
 
 gchar*
 gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
 {
   gchar*     retval = 0;
   
+  g_return_val_if_fail(IS_GDA_FIELD(f), 0);
+  
   if (bfr)
     retval = bfr;
 
@@ -222,7 +311,7 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
 	  maxlen = 128;
 	}
       printf("stringify for valuetype [%d]'%s' NYI\n",
-	     f->attributes->gdaType, gda_fieldtype_2_string(0, 20, f));
+	     f->attributes->gdaType, gda_fieldtype_2_string(0, 20, gda_field_type(f)));
       retval[0] = '\0';
       break;
     }
@@ -237,10 +326,10 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
  */
 
 Gda_Field*
-gda_field_new(void)
+gda_field_new (void)
 {
-  Gda_Field* rc = g_new0(Gda_Field,1);
-  return rc;
+  Gda_Field* rc = GDA_FIELD(gtk_type_new(gda_field_get_type()));
+  return (rc);
 }
 
 
@@ -253,8 +342,9 @@ gda_field_new(void)
  */
 
 void
-gda_field_free(Gda_Field* f)
+gda_field_free (Gda_Field* f)
 {
+  g_return_if_fail(IS_GDA_FIELD(f));
   g_free(f);
 }
 
@@ -270,10 +360,11 @@ gda_field_free(Gda_Field* f)
  */
 
 gint
-gda_field_actual_size(Gda_Field* f)
+gda_field_actual_size (Gda_Field* f)
 {
-  if (!f || gda_field_isnull(f))
-    return 0;
+  g_return_val_if_fail(IS_GDA_FIELD(f), 0);
+  
+  if (gda_field_isnull(f)) return 0;
   
   switch (gda_field_type(f))
     {
@@ -321,8 +412,10 @@ gda_field_actual_size(Gda_Field* f)
       return sizeof(CORBA_unsigned_long_long);
     case GDA_TypeUSmallint:
       return sizeof(CORBA_unsigned_short);
+    default:
+      g_warning("gda_field_actual_size: unknown GDA Type %d\n", gda_field_type(f));
+      return -1;
     }
   g_warning("gda_field_actual_size: unknown GDA Type %d\n", gda_field_type(f));
   return -1;
-};
-  
+}
