@@ -1,5 +1,6 @@
 /* GNOME DB libary
  * Copyright (C) 1998,1999 Michael Lausch
+ * Copyright (C) 1999 Rodrigo Moya
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,11 +20,27 @@
 #include <gnome.h>
 #include <gda.h>
 
-
 #include "gda-recordset.h"
 #include "gda-command.h"
+#include "gda-commandP.h"
 #include "gda-error.h"
 
+enum
+{
+  RECORDSET_ERROR,
+  RECORDSET_EOF,
+  RECORDSET_BOF,
+  LAST_SIGNAL
+};
+
+static gint gda_recordset_signals[LAST_SIGNAL] = {0, };
+
+static void   gda_recordset_class_init (Gda_RecordsetClass *);
+static void   gda_recordset_init       (Gda_Recordset *);
+static void   gda_recordset_real_error (Gda_Recordset *, GList *);
+static gulong fetch_and_store          (Gda_Recordset* rs, gint count, gpointer bookmark);
+static gulong fetch_and_dont_store     (Gda_Recordset* rs, gint count, gpointer bookmark);
+
 /* per default there's no upper limit on the number of rows which can
  * be fetched from the data source
  */
@@ -32,31 +49,164 @@
 /* per default fetch 64 rows at a time */
 #define GDA_DEFAULT_CACHESIZE 64
 
-
-
-/*
- * this function will change as soon as the client side
- * error objects are here
- */
-static gint
-Exception(CORBA_Environment *ev)
-{
-  switch (ev->_major)
-    {
-    case CORBA_SYSTEM_EXCEPTION:
-      fprintf(stderr,"CORBA system exception %s.\n", CORBA_exception_id(ev));
-      return -1;
-    case CORBA_USER_EXCEPTION:
-      fprintf(stderr,"CORBA user exception %s.\n", CORBA_exception_id(ev));
-      return -1;
-    default:
-    }
-  return 0;
-}
-
+#define __DEBUG_NEW_CODE 1
 
 static void
-free_chunks(GList* chunks)
+gda_recset_value_changed (GtkAdjustment* adj)
+{
+  Gda_Recordset* rs = GDA_RECORDSET(adj);
+  gint           offset;
+
+#if __DEBUG_NEW_CODE
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+  g_print("\tadj->value  = %g\n", adj->value);
+  g_print("\trs->current = %ld\n", rs->current_index);
+
+#endif
+
+  if (adj->value < 1.0)
+    adj->value = 1.0;
+  
+  if (adj->value == G_MAXFLOAT)
+    offset = G_MAXINT;
+  else
+    offset = adj->value - rs->current_index;
+
+  if (rs->cursor_type == GDA_OPEN_FWDONLY && offset < 0)
+    return;
+
+  if (offset == 0)
+    {
+#if __DEBUG_NEW_CODE
+      g_print("\toffset == 0, No action taken\n");
+#endif      
+      return;
+    }
+#if __DEBUG_NEW_CODE
+    g_print("\toffset = %d\n", (gint)offset);
+#endif
+    
+  if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
+    fetch_and_store(rs, offset, 0);
+  else
+    fetch_and_dont_store(rs, offset, 0);
+
+  adj->value = rs->current_index;
+
+#if __DEBUG_NEW_CODE
+  g_print("\tnew adjustment value = %g\n", adj->value);
+#endif;  
+  if (gda_recordset_eof(rs))
+    {
+      adj->upper = adj->value;
+    }
+}
+
+static void
+gda_recordset_real_error (Gda_Recordset *rs, GList *error_list)
+{
+  Gda_Connection *cnc;
+#if __DEBUG_NEW_CODE
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+#endif
+  if ((cnc = gda_recordset_get_connection(rs)))
+    {
+      gda_connection_add_errorlist(cnc, error_list);
+    }
+}
+
+guint
+gda_recordset_get_type (void)
+{
+  static guint gda_recordset_type = 0;
+
+  if (!gda_recordset_type)
+    {
+      GtkTypeInfo gda_recordset_info =
+      {
+	"GdaRecordset",
+	sizeof(Gda_Recordset),
+	sizeof(Gda_RecordsetClass),
+	(GtkClassInitFunc) gda_recordset_class_init,
+	(GtkObjectInitFunc) gda_recordset_init,
+	(GtkArgSetFunc) NULL,
+	(GtkArgSetFunc) NULL
+      };
+      gda_recordset_type = gtk_type_unique(gtk_adjustment_get_type(), &gda_recordset_info);
+    }
+  return (gda_recordset_type);
+}
+
+typedef void (*GtkSignal_NONE__INT_POINTER)(GtkObject,
+					    guint  arg1,
+					    gpointer arg2,
+					    gpointer user_data);
+
+static void
+gda_recordset_class_init (Gda_RecordsetClass *klass)
+{
+  GtkObjectClass       *object_class = (GtkObjectClass *) klass;
+  GtkAdjustmentClass*   adjustment_class = (GtkAdjustmentClass*)klass;
+  
+  gda_recordset_signals[RECORDSET_ERROR] = gtk_signal_new("error", 
+							  GTK_RUN_LAST,
+							  object_class->type,
+							  GTK_SIGNAL_OFFSET(Gda_RecordsetClass, error),
+							  gtk_marshal_NONE__POINTER,
+							  GTK_TYPE_NONE,
+							  1,
+							  GTK_TYPE_POINTER);
+  gda_recordset_signals[RECORDSET_EOF] = gtk_signal_new("eof",
+							GTK_RUN_LAST,
+							object_class->type,
+							GTK_SIGNAL_OFFSET(Gda_RecordsetClass, eof),
+							gtk_signal_default_marshaller,
+							GTK_TYPE_NONE, 0);
+  gda_recordset_signals[RECORDSET_BOF] = gtk_signal_new("bof",
+							GTK_RUN_LAST,
+							object_class->type,
+							GTK_SIGNAL_OFFSET(Gda_RecordsetClass, bof),
+							gtk_signal_default_marshaller,
+							GTK_TYPE_NONE, 0);
+
+  gtk_object_class_add_signals(object_class, gda_recordset_signals, LAST_SIGNAL);
+
+  klass->error = gda_recordset_real_error;
+  klass->eof   = 0;
+  klass->bof   = 0;
+  adjustment_class->value_changed = gda_recset_value_changed;
+}
+
+static void
+gda_recordset_init (Gda_Recordset *rs)
+{
+
+  rs->external_cmd     = 0;
+  rs->internal_cmd     = 0;
+  rs->corba_rs         = CORBA_OBJECT_NIL;
+  rs->cnc              = 0;
+  rs->chunks           = 0;
+  rs->chunks_length    = 0;
+  rs->current_row      = 0;
+  rs->field_attributes = 0;
+  rs->max_index        = 0;
+  rs->maxrows          = 0;
+  rs->cachesize        = 0;
+  rs->open             = 0;
+  rs->eof              = 1;
+  rs->bof              = 1;
+  rs->readonly         = 1;
+  rs->cursor_location  = GDA_USE_CLIENT_CURSOR;
+  rs->cursor_type      = GDA_OPEN_FWDONLY;
+  rs->name             = 0;
+  
+  GTK_ADJUSTMENT(rs)->upper = G_MAXFLOAT;
+  GTK_ADJUSTMENT(rs)->lower = 1;
+  GTK_ADJUSTMENT(rs)->value = 0;
+}
+
+static void
+free_chunks (GList* chunks)
 {
   GDA_Row* row;
   GList*   ptr;
@@ -71,18 +221,17 @@ free_chunks(GList* chunks)
 }
 
 static GDA_Row*
-row_by_idx(Gda_Recordset* rs, gint idx)
+row_by_idx (Gda_Recordset* rs, gint idx)
 {
   GDA_Recordset_Chunk* chunk;
 
-  if (idx < 0 || idx > g_list_length(rs->chunks))
+  if (idx <= 0 || idx > g_list_length(rs->chunks))
     return 0;
-  chunk = g_list_nth(rs->chunks, idx)->data;
+  chunk = g_list_nth(rs->chunks, idx-1)->data;
 
   return &chunk->_buffer[0];
 }
 
-
 /**
  * gda_recordset_new:
  *
@@ -91,26 +240,15 @@ row_by_idx(Gda_Recordset* rs, gint idx)
  * Returns: the allocated recordset object
  */
 
-Gda_Recordset*
-gda_recordset_new(void)
+GtkObject*
+gda_recordset_new (void)
 {
   Gda_Recordset* rs;
 
-  rs = g_new0(Gda_Recordset, 1);
-  rs->maxrows = GDA_DEFAULT_MAXROWS;
-  rs->cachesize = GDA_DEFAULT_CACHESIZE;
-  rs->readonly = 1;
-  rs->chunks_length = 0;
-  rs->open = 0;
-  rs->eof = rs->bof = 1;
-  rs->cursor_location = GDA_USE_CLIENT_CURSOR;
-  rs->cursor_type = GDA_OPEN_FWDONLY;
-  rs->max_index = -1;
-  rs->current_index = -1;
-  return rs;
+  rs = gtk_type_new(gda_recordset_get_type());
+  return GTK_OBJECT(rs);
 }
 
-
 /**
  * gda_recordset_free:
  * @rs: the recordset which should be destroyed.
@@ -120,8 +258,10 @@ gda_recordset_new(void)
  *
  */
 void
-gda_recordset_free(Gda_Recordset* rs)
+gda_recordset_free (Gda_Recordset* rs)
 {
+  g_return_if_fail(IS_GDA_RECORDSET(rs));
+  
   if (rs->open)
     gda_recordset_close(rs);
   if (rs->internal_cmd)
@@ -131,7 +271,6 @@ gda_recordset_free(Gda_Recordset* rs)
   g_free(rs);
 }
 
-
 /**
  * gda_recordset_close:
  * @rs: the recordset to close
@@ -145,10 +284,12 @@ gda_recordset_free(Gda_Recordset* rs)
  *
  */
 void
-gda_recordset_close(Gda_Recordset* rs)
+gda_recordset_close (Gda_Recordset* rs)
 {
   CORBA_Environment ev;
 
+  g_return_if_fail(IS_GDA_RECORDSET(rs));
+  
   CORBA_exception_init(&ev);
   
   if (!rs->open)
@@ -157,7 +298,7 @@ gda_recordset_close(Gda_Recordset* rs)
   if (rs->corba_rs)
     {
       GDA_Recordset_close(rs->corba_rs, &ev);
-      Exception(&ev);
+      gda_connection_corba_exception(rs->cnc, &ev);
     }
   rs->corba_rs = CORBA_OBJECT_NIL;
   if (rs->chunks)
@@ -167,7 +308,6 @@ gda_recordset_close(Gda_Recordset* rs)
   rs->chunks = 0;
 }
 
-
 /**
  * gda_recordset_bof:
  * @rs: Recordset which should be checked
@@ -183,12 +323,13 @@ gda_recordset_close(Gda_Recordset* rs)
 gint
 gda_recordset_bof(Gda_Recordset* rs)
 {
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
+  
   if (!rs->open || !rs->current_row || rs->bof)
     return 1;
   return 0;
 }
 
-
 /**
  * gda_recordset_eof:
  * @rs:  Recordset which should be checked
@@ -202,14 +343,15 @@ gda_recordset_bof(Gda_Recordset* rs)
  * recordset is empty, 0 otherwise.
  */
 gint
-gda_recordset_eof(Gda_Recordset* rs)
+gda_recordset_eof (Gda_Recordset* rs)
 {
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
+  
   if (!rs->open || rs->eof)
     return 1;
   return 0;
 }
 
-
 static gulong
 fetch_and_store(Gda_Recordset* rs, gint count, gpointer bookmark)
 {
@@ -243,7 +385,7 @@ fetch_and_store(Gda_Recordset* rs, gint count, gpointer bookmark)
   for (current_count = 0; current_count < count; current_count++)
     {
       chunk = GDA_Recordset_fetch(rs->corba_rs, 1, &ev);
-      if (Exception(&ev))
+      if (gda_connection_corba_exception(rs->cnc, &ev))
 	{
 	  /* An error */
 	  return rs->current_index;
@@ -277,7 +419,8 @@ fetch_and_dont_store(Gda_Recordset* rs, gint count, gpointer bookmark)
   GDA_Recordset_Chunk* chunk;
   gulong               current_idx;
   CORBA_Environment    ev;
-
+  GList*               error_list;
+  
   current_idx = rs->current_index;
   current_idx += count;
   if (current_idx < 0)
@@ -285,16 +428,27 @@ fetch_and_dont_store(Gda_Recordset* rs, gint count, gpointer bookmark)
   
   CORBA_exception_init(&ev);
   GDA_Recordset_moveFirst(rs->corba_rs, &ev);
-  if (Exception(&ev))
-    return GDA_RECORDSET_INVALID_POSITION;
+  error_list = gda_errors_from_exception(&ev);
+  if (error_list)
+    {
+      gda_connection_add_errorlist(rs->cnc, error_list);
+      return GDA_RECORDSET_INVALID_POSITION;
+    }
   
   GDA_Recordset_move(rs->corba_rs, current_idx-1, 0, &ev);
-  if (Exception(&ev))
-    return GDA_RECORDSET_INVALID_POSITION;
-
+  error_list = gda_errors_from_exception(&ev);
+  if (error_list)
+    {
+      gda_connection_add_errorlist(rs->cnc, error_list);
+      return GDA_RECORDSET_INVALID_POSITION;
+    }
   chunk = GDA_Recordset_fetch(rs->corba_rs, count, &ev);
-  if (Exception(&ev))
-    return GDA_RECORDSET_INVALID_POSITION;
+  error_list = gda_errors_from_exception(&ev);
+  if (error_list)
+    {
+      gda_connection_add_errorlist(rs->cnc, error_list);
+      return GDA_RECORDSET_INVALID_POSITION;
+    }
 
   if (chunk->_length == 0)
     {
@@ -312,7 +466,6 @@ fetch_and_dont_store(Gda_Recordset* rs, gint count, gpointer bookmark)
   return rs->current_index;
 }
 
-
 /**
  * gda_recordset_move:
  * @rs: the recordset
@@ -340,31 +493,16 @@ fetch_and_dont_store(Gda_Recordset* rs, gint count, gpointer bookmark)
  * fetching the rows.  
  */
 gulong
-gda_recordset_move(Gda_Recordset* rs, gint count, gpointer bookmark)
+gda_recordset_move (Gda_Recordset* rs, gint count, gpointer bookmark)
 {
-
-  gulong               rc;
-  
-
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), GDA_RECORDSET_INVALID_POSITION);
   g_return_val_if_fail(rs->corba_rs != NULL, GDA_RECORDSET_INVALID_POSITION);
   g_return_val_if_fail(rs->open, GDA_RECORDSET_INVALID_POSITION);
   
-  if (count == 0)
-    {
-      return rs->current_index;
-    }
-
-  if (rs->cursor_type == GDA_OPEN_FWDONLY && count < 0)
-    return GDA_RECORDSET_INVALID_POSITION;
-  
-  if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
-    rc = fetch_and_store(rs, count, bookmark);
-  else
-    rc = fetch_and_dont_store(rs, count, bookmark);
+  gtk_adjustment_set_value(GTK_ADJUSTMENT(rs), rs->current_index + count);
   return rs->current_index;
 }
 
-
 /**
  * gda_recordset_move_first:
  * @rs: the recordset
@@ -377,18 +515,18 @@ gda_recordset_move(Gda_Recordset* rs, gint count, gpointer bookmark)
  * %GDA_RECORDSET_INVALID_POSITION
  * if there was an error. 
  */
-
 gulong
-gda_recordset_move_first(Gda_Recordset* rs)
+gda_recordset_move_first (Gda_Recordset* rs)
 {
 
-  g_return_val_if_fail(rs != 0, -1);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), GDA_RECORDSET_INVALID_POSITION);
+  
   if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
     {
       GDA_Recordset_Chunk* chunk;
       if (!rs->chunks)
 	return gda_recordset_move(rs, 1, 0);
-      rs->current_index = 0;
+      rs->current_index = 1;
       chunk = rs->chunks->data;
       rs->current_row = &chunk->_buffer[0];
       rs->eof = 0;
@@ -396,8 +534,8 @@ gda_recordset_move_first(Gda_Recordset* rs)
     }
   g_error("gda_recordset_move_first for server based cursor NIY");
   return GDA_RECORDSET_INVALID_POSITION;
-}
 
+}
 
 /**
  * gda_recordset_move_last:
@@ -411,13 +549,14 @@ gda_recordset_move_first(Gda_Recordset* rs)
  * %GDA_RECORDSET_INVALID_POSITION
  * if there was an error. 
  */
-
 gulong
-gda_recordset_move_last(Gda_Recordset* rs)
+gda_recordset_move_last (Gda_Recordset* rs)
 {
   gulong pos;
   gulong rc;
   
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), GDA_RECORDSET_INVALID_POSITION);
+  
   if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
     {
       pos = rs->current_index;
@@ -436,7 +575,6 @@ gda_recordset_move_last(Gda_Recordset* rs)
   return GDA_RECORDSET_INVALID_POSITION;
 }
 
-
 /**
  * gda_recordset_move_next:
  * @rs: the recordset
@@ -449,11 +587,11 @@ gda_recordset_move_last(Gda_Recordset* rs)
  * %GDA_RECORDSET_INVALID_POSITION
  * if there was an error. 
  */
-
 gulong
-gda_recordset_move_next(Gda_Recordset* rs)
+gda_recordset_move_next (Gda_Recordset* rs)
 {
-
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), GDA_RECORDSET_INVALID_POSITION);
+  
   return gda_recordset_move(rs, 1, 0);
 }
 
@@ -468,15 +606,13 @@ gda_recordset_move_next(Gda_Recordset* rs)
  * Returns: a pointer to a field structor or NULL if 
  * @idx is out of range
  */
-
 Gda_Field*
-gda_recordset_field_idx(Gda_Recordset* rs, gint idx)
+gda_recordset_field_idx (Gda_Recordset* rs, gint idx)
 {
-
-  gint        rowsize;
-  Gda_Field*  rc;
+  gint       rowsize;
+  Gda_Field* rc;
   
-  g_return_val_if_fail(rs != NULL, 0);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
   g_return_val_if_fail(idx >= 0, 0);
   g_return_val_if_fail(rs->open, 0);
   
@@ -510,15 +646,13 @@ gda_recordset_field_idx(Gda_Recordset* rs, gint idx)
  * Returns: a pointer to a field structor or NULL if 
  * @idx is out of range
  */
-
 Gda_Field*
-gda_recordset_field_name(Gda_Recordset* rs, gchar* name)
+gda_recordset_field_name (Gda_Recordset* rs, gchar* name)
 {
-
   gint        rowsize;
   gint        idx;
   
-  g_return_val_if_fail(rs != NULL, 0);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
   g_return_val_if_fail(rs->open, 0);
   
   if (!rs->current_row && !rs->field_attributes)
@@ -546,9 +680,9 @@ gda_recordset_field_name(Gda_Recordset* rs, gchar* name)
  * 0 if there was an error.
  */
 gint
-gda_recordset_rowsize(Gda_Recordset* rs)
+gda_recordset_rowsize (Gda_Recordset* rs)
 {
-  g_return_val_if_fail(rs != NULL, -1);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
   g_return_val_if_fail(rs->open, -1);
 
   if (!rs->current_row && !rs->field_attributes)
@@ -574,8 +708,10 @@ gda_recordset_rowsize(Gda_Recordset* rs)
  */
 
 gint
-gda_recordset_set_connection(Gda_Recordset* rs, Gda_Connection* cnc)
+gda_recordset_set_connection (Gda_Recordset* rs, Gda_Connection* cnc)
 {
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
+  
   if (rs->open)
     return -1;
   rs->cnc = cnc;
@@ -591,32 +727,73 @@ gda_recordset_set_connection(Gda_Recordset* rs, Gda_Connection* cnc)
  * this recordset.
  */
 Gda_Connection*
-gda_recordset_get_connection(Gda_Recordset* rs)
+gda_recordset_get_connection (Gda_Recordset* rs)
 {
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
+  
   if (!rs->open)
     return 0;
   return rs->cnc;
 }
 
-
-static void
-add_errors_to_list(Gda_Recordset* rs, GDA_DriverError* driver_errors)
+/**
+ * gda_recordset_open:
+ * @rs: the recordset
+ * @cmd: the command object
+ * @cursor_type: the type of the cursor used. Currently only a forward 
+ * only cursor is implemented
+ * @lock_type: the type of locks used for the command. Currently only
+ * readonly type of locking is supported.
+ * @options: currently not implemented. Used to identify how to parse
+ * the @txt.
+ *
+ * This function opens a recordset. The recordset is filled with the
+ * output of @cmd. Before this function is called various parameters
+ * of the recordst might be changed. This is the most featurefull
+ * function for retrieving results from a database.
+ *
+ * Returns: 0 if everytrhing is okay, -1 on error
+ */
+gint
+gda_recordset_open (Gda_Recordset* rs,
+                    Gda_Command* cmd,
+                    GDA_CursorType cursor_type,
+		    GDA_LockType lock_type,
+		    gulong options)
 {
-  gint idx;
+  CORBA_Environment  ev;
+  GDA_CmdParameterSeq*      corba_parameters;
+  CORBA_long                affected = 0;
+  GList*                    error_list;
 
-  for (idx = 0; idx < driver_errors->errors._length; idx++)
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
+  g_return_val_if_fail(!rs->open, -1);
+
+  gda_recordset_init(rs);
+  corba_parameters = __gda_command_get_params(cmd);
+  rs->cursor_type = cursor_type;
+  rs->lock_type   = lock_type;
+  CORBA_exception_init(&ev);
+  rs->cnc = cmd->connection;
+  rs->corba_rs = GDA_Command_open(cmd->command, corba_parameters,
+				  cursor_type, lock_type, &affected, &ev);
+  error_list = gda_errors_from_exception(&ev);
+  if (error_list)
     {
-      Gda_Error* e;
-      e = gda_error_new();
-      e->description = g_strdup(driver_errors->errors._buffer[idx].description);
-      e->number      = driver_errors->errors._buffer[idx].number;
-      e->source      = g_strdup(driver_errors->errors._buffer[idx].source);
-      e->helpurl = 0;
-      e->sqlstate    = g_strdup(driver_errors->errors._buffer[idx].sqlstate);
-      e->native      = g_strdup(driver_errors->errors._buffer[idx].nativeMsg);
-      rs->cnc->errors = g_list_append(rs->cnc->errors, e);
+      rs->corba_rs = 0;
+      gda_connection_add_errorlist(rs->cnc, error_list);
+      return -1;
     }
-  CORBA_free(driver_errors);
+
+  rs->field_attributes = GDA_Recordset_describe(rs->corba_rs, &ev);
+  error_list = gda_errors_from_exception(&ev);
+  if (error_list)
+    {
+      gda_connection_add_errorlist(rs->cnc, error_list);
+      return -1;
+    }
+  rs->open = 1;
+  return 0;
 }
 
 /**
@@ -630,69 +807,44 @@ add_errors_to_list(Gda_Recordset* rs, GDA_DriverError* driver_errors)
  * @options: currently not implemented. Used to identify how to parse
  * the @txt.
  *
- * This function opens a recordsrt. The recordset is filled with the
+ * This function opens a recordset. The recordset is filled with the
  * output of @txt. Before this function is called various parameters
- * ofthe recordst might be changed. This is the most featurefull
+ * of the recordst might be changed. This is the most featurefull
  * function for retrieving results from a database.
  *
  * Returns: 0 if everytrhing is okay, -1 on error
  */
 gint
-gda_recordset_open_txt(Gda_Recordset* rs, gchar* txt, GDA_CursorType cursor_type,
-		       GDA_LockType lock_type, gulong options)
+gda_recordset_open_txt (Gda_Recordset* rs,
+                        gchar* txt,
+                        GDA_CursorType cursor_type,
+		        GDA_LockType lock_type,
+		        gulong options)
 {
   CORBA_Environment  ev;
+  Gda_Command*       cmd;
+  gint               rc;
+  
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
   
   if (rs->open)
     return -1;
   rs->cursor_type = cursor_type;
   rs->lock_type   = lock_type;
   CORBA_exception_init(&ev);
-  rs->corba_rs = GDA_Connection_openRecordset(rs->cnc->connection, txt,
-					      lock_type, cursor_type, options, &ev);
-  if (Exception(&ev))
-    {
-      if (strcmp(CORBA_exception_id(&ev), ex_GDA_DriverError) == 0)
-	{
-	  add_errors_to_list(rs, (GDA_DriverError*)ev._params);
-	}
-      if (rs->corba_rs)
-	{
-	  rs->corba_rs = 0;
-	}
-    }
-  if (!rs->corba_rs)
-    return -1;
-
-  rs->field_attributes = GDA_Recordset_describe(rs->corba_rs, &ev);
-
-  if (Exception(&ev) || rs->field_attributes == CORBA_OBJECT_NIL)
-    {
-      if (strcmp(CORBA_exception_id(&ev), ex_GDA_DriverError) == 0)
-	{
-	  add_errors_to_list(rs, (GDA_DriverError*)ev._params);
-	}
-      if (rs->field_attributes)
-	{
-	  CORBA_free(rs->field_attributes);
-	  rs->field_attributes = 0;
-	}
-      return -1;
-    }
-  rs->open = 1;
-  rs->current_index = -1;
-  rs->max_index = -1;
-  rs->current_row = 0;
-  rs->eof = 1;
-  rs->bof = 1;
-  return 0;
+  cmd = gda_command_new();
+  gda_command_set_connection(cmd, rs->cnc);
+  gda_command_set_text(cmd, txt);
+  rc = gda_recordset_open(rs, cmd, cursor_type, lock_type, options);
+  gda_command_free(cmd);
+  return rc;
 }
 
 gint
-gda_recordset_add_field(Gda_Recordset* rs, Gda_Field* field)
+gda_recordset_add_field (Gda_Recordset* rs, Gda_Field* field)
 {
-  g_return_val_if_fail(rs, -1);
-  g_return_val_if_fail(field, -1);
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
+  g_return_val_if_fail(IS_GDA_FIELD(field), -1);
   g_return_val_if_fail(field->attributes, -1);
   g_return_val_if_fail(field->attributes->name, -1);
 
@@ -724,8 +876,6 @@ gda_recordset_add_field(Gda_Recordset* rs, Gda_Field* field)
   return 0;
 }
 
-
-
 /**
  * gda_recordset_get_cursorloc:
  * @rs: the recordset
@@ -749,6 +899,8 @@ gda_recordset_get_cursorloc(Gda_Recordset* rs)
 void
 gda_recordset_set_cursorloc(Gda_Recordset* rs, GDA_CursorLocation loc)
 {
+  g_return_if_fail(IS_GDA_RECORDSET(rs));
+  
   rs->cursor_location = loc;
 }
 
@@ -761,6 +913,7 @@ gda_recordset_set_cursorloc(Gda_Recordset* rs, GDA_CursorLocation loc)
 GDA_CursorType
 gda_recordset_get_cursortype(Gda_Recordset* rs)
 {
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), -1);
   return rs->cursor_type;
 }
 
@@ -775,6 +928,8 @@ gda_recordset_get_cursortype(Gda_Recordset* rs)
 void
 gda_recordset_set_cursortype(Gda_Recordset* rs, GDA_CursorType type)
 {
+  g_return_if_fail(IS_GDA_RECORDSET(rs));
+  
   rs->cursor_type = type;
 }
 
