@@ -1,5 +1,6 @@
 /* GNOME DB libary
  * Copyright (C) 1998,1999 Michael Lausch
+ * Copyright (C) 2000 Rodrigo Moya
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,12 +28,19 @@ typedef struct _Gda_Server
   gchar* comment;
   gchar* location;
   gchar* repo_id;
-}Gda_Server;
+} Gda_Server;
+
+#define GDA_SERVER_GOAD_TYPE(srv) ((srv) ? (srv)->type : 0)
+#define GDA_SERVER_NAME(srv)      ((srv) ? (srv)->name : 0)
+#define GDA_SERVER_COMMENT(srv)   ((srv) ? (srv)->comment : 0)
+#define GDA_SERVER_LOCATION(srv)  ((srv) ? (srv)->location : 0)
+#define GDA_SERVER_REPO_ID(srv)   ((srv) ? (srv)->repo_id : 0)
+
+Gda_Server* gda_server_new                    (void);
+Gda_Server* gda_server_copy                   (Gda_Server*);
+void        gda_server_free                   (Gda_Server*);
+GList*      gda_server_list                   (void);
+GList*      gda_list_datasources              (void);
+GList*      gda_list_datasources_for_provider (gchar* provider);
 
-Gda_Server*                gda_server_new                   (void);
-Gda_Server*                gda_server_copy                  (Gda_Server*);
-void                       gda_server_free                  (Gda_Server*);
-GList*                     gda_server_list                  (void);
-GList*                     gda_list_datasources             (void);
-GList*                     gda_list_datasources_for_provider(gchar* provider);
 #endif
