@@ -29,7 +29,8 @@
 static       GList*            providers = 0;
 static       GtkWidget*        current_tree;
 static       GtkWidget*        provider_pixmap = 0;
-static       GtkWidget*        name_pixmap = 0;
+static       GtkWidget*        clean_name_pixmap = 0;
+static       GtkWidget*        dirty_name_pixmap = 0;
 static       Gdamgr_NodeInfo*  current_node = 0;
 
 
@@ -222,10 +223,6 @@ gdamgr_entries_list(void)
   return retval;
 }
 
-
-
-  
-
 Gdamgr_NodeInfo*
 gdamgr_get_selected_node(void)
 {
@@ -371,6 +368,51 @@ accept_provider_changes(GtkWidget* button, gpointer data)
   gtk_widget_show(msg);
 }
 
+void
+entry_edited(GtkWidget* entry, gpointer data)
+{
+  Gdamgr_EntryInfo* entry_info;
+  Gdamgr_NodeInfo*  node = gdamgr_get_selected_node();
+  gchar* txt;
+  
+  g_print("entry_changed for %p called\n", node);
+  if (node->ntype != EntryNode)
+    {
+      g_error("accept_entry_changes: Inconsistency detected");
+    }
+  if (!GTK_WIDGET_HAS_FOCUS(entry))
+    {
+      g_print("Entry has no focus\n");
+      return;
+    }
+  entry_info = (Gdamgr_EntryInfo*)node->node;
+  if (!entry_info->edited)
+    {
+      txt = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+      entry_info->edited = 1;
+      gdamgr_refresh_provider_tree(current_tree);
+      gtk_entry_set_text(GTK_ENTRY(entry), txt);
+      g_free(txt);
+    }
+}
+  
+gint
+entry_activated(GtkWidget* entry, gpointer data)
+{
+  Gdamgr_NodeInfo*  node = gdamgr_get_selected_node();
+  g_print("entry_activated for %p called\n", node);
+  return 0;
+}
+
+gint
+entry_deactivated(GtkWidget* entry, gpointer data)
+{
+  Gdamgr_NodeInfo*  node = gdamgr_get_selected_node();
+
+  g_print("entry_deactivated for %p called\n", node);
+
+  return 0;
+}
 
 void
 accept_entry_changes(GtkWidget* button, gpointer data)
@@ -433,10 +475,12 @@ accept_entry_changes(GtkWidget* button, gpointer data)
       need_refresh = 1;
     }
   if (need_refresh)
-    gdamgr_refresh_provider_tree(current_tree);
+    {
+      gdamgr_refresh_provider_tree(current_tree);
+      need_save = 1;
+    }
 }
 
-
 static void
 remove_node(GtkCTree* tree, GtkCTreeNode* node, gpointer data)
 {
@@ -447,8 +491,6 @@ remove_node(GtkCTree* tree, GtkCTreeNode* node, gpointer data)
   gtk_ctree_remove_node(GTK_CTREE(tree), node);
 }
 
-    
-
 void
 gdamgr_refresh_provider_tree(GtkWidget* tree)
 {
@@ -474,10 +516,13 @@ gdamgr_refresh_provider_tree(GtkWidget* tree)
   gtk_ctree_post_recursive(GTK_CTREE(tree), 0, remove_node, 0);
 
   if (!provider_pixmap)
-    provider_pixmap = gnome_pixmap_new_from_file("gda-servers.png");
-  if (!name_pixmap)
-    name_pixmap = gnome_pixmap_new_from_file("gda-names.png");
-  
+    provider_pixmap =
+      gnome_pixmap_new_from_file(gnome_pixmap_file("gnome-db/gda-servers.png"));
+  if (!clean_name_pixmap)
+    {
+      clean_name_pixmap = gnome_pixmap_new_from_file(gnome_pixmap_file("gnome-db/db-black.png"));
+      dirty_name_pixmap = gnome_pixmap_new_from_file(gnome_pixmap_file("gnome-db/db-red.png"));
+    }
   providers_ptr = providers;
   parent = 0;
   sibling = 0;
@@ -512,7 +557,11 @@ gdamgr_refresh_provider_tree(GtkWidget* tree)
       sibling_name = 0;
       while (gda_entries_ptr)
 	{
+	  GtkWidget* name_pixmap;
+
 	  gda_entry = gda_entries_ptr->data;
+	  name_pixmap = gda_entry->edited ? dirty_name_pixmap : clean_name_pixmap;
+	  
 	  text[0] = gda_entry->gda_name;
 	  sibling = gtk_ctree_insert_node(GTK_CTREE(tree),
 					  parent, sibling,
@@ -555,8 +604,6 @@ gdamgr_refresh_provider_tree(GtkWidget* tree)
   gtk_clist_thaw(GTK_CLIST(tree));
 }
 
-
-
 void
 new_provider_cmd(void)
 {
@@ -566,11 +613,65 @@ new_provider_cmd(void)
 void
 new_datasource_cmd(void)
 {
-  ;
+  Gdamgr_EntryInfo* new_entry;
+  Gdamgr_ProviderInfo* provider;
+  static GtkWidget* gda_prompt_box = 0;
+  static GtkWidget* gda_name_entry = 0;
+  
+  if (!current_node)
+    {
+      static GtkWidget* box = 0;
+
+      if (!box)
+	{
+	  box = gnome_message_box_new(_("No Provider selected"),
+				      "error");
+	  gtk_window_set_modal(GTK_WINDOW(box), TRUE);
+	  gnome_dialog_close_hides(GNOME_DIALOG(box), TRUE);
+	}
+      gtk_widget_show(box);
+      return;
+    }
+  if (!gda_prompt_box)
+    {
+      GtkWidget* label;
+      GtkWidget* table;
+      
+      gda_prompt_box = gnome_dialog_new(_("Enter GDA Name"),
+					GNOME_STOCK_BUTTON_OK, NULL);
+      table = gtk_table_new(1, 2, FALSE);
+      label = gtk_label_new(_("GDA Name"));
+      gtk_misc_set_alignment(GTK_MISC(label), 0.1, 1.0);
+      gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
+		       GTK_FILL, 0, 0, 0);
+      gda_name_entry = gtk_entry_new();
+      gtk_table_attach(GTK_TABLE(table), gda_name_entry, 1, 2, 0, 1,
+		       GTK_FILL, 0, 0, 0);
+      gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(gda_prompt_box)->vbox),
+			 table, TRUE, TRUE, GNOME_PAD);
+      gnome_dialog_close_hides(GNOME_DIALOG(gda_prompt_box), TRUE);
+      gnome_dialog_set_close(GNOME_DIALOG(gda_prompt_box), TRUE);
+    }
+  gtk_widget_show_all(gda_prompt_box);
+  gnome_dialog_run(GNOME_DIALOG(gda_prompt_box));
+  new_entry = g_new0(Gdamgr_EntryInfo, 1);
+  new_entry->gda_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(gda_name_entry)));
+  new_entry->dsn = g_strdup("DSN");
+  new_entry->description = g_strdup("Description");
+  new_entry->configurator = g_strdup("None");
+  new_entry->expanded = 0;
+  new_entry->selected = 1;
+  if (current_node->ntype == ProviderNode)
+    {
+      provider = current_node->node;
+      provider->selected = 0;
+    }
+  else
+    provider = ((Gdamgr_EntryInfo*)current_node->node)->provider;
+  gdamgr_provider_add_datasource(provider, new_entry);
+  gdamgr_refresh_provider_tree(current_tree);
 }
 
-
-
 void
 save_cmd(void)
 {
@@ -581,6 +682,23 @@ save_cmd(void)
 void
 quit_cmd(void)
 {
+  gint cancel;
+  
+  GtkWidget* save_box;
+
+
+  if (need_save)
+    {
+      save_box = gnome_message_box_new(_("You have edited the GDA configuration file.\n\nSave the file?"),
+				       GNOME_MESSAGE_BOX_QUESTION,
+				       GNOME_STOCK_BUTTON_OK,
+				       GNOME_STOCK_BUTTON_CANCEL,
+				       NULL);
+      gnome_message_box_set_modal(GNOME_MESSAGE_BOX(save_box));
+      cancel = gnome_dialog_run(GNOME_DIALOG(save_box));
+      if (!cancel)
+	gdamgr_store_gda_info();
+    }
   gtk_main_quit();
 }
 
@@ -686,7 +804,7 @@ gdamgr_store_gda_info(void)
   GString*             path;
   Gdamgr_EntryInfo*    entry;
 
-  gnome_config_push_prefix("/gdalib/Datasources");
+  gnome_config_push_prefix("/gdalib/Datasources/");
   datasources_ptr = datasources = gdamgr_entries_list();
   while (datasources_ptr)
     {
@@ -694,7 +812,6 @@ gdamgr_store_gda_info(void)
       entry = datasources_ptr->data;
       gnome_config_set_string(entry->gda_name, entry->gda_name);
       datasources_ptr = g_list_next(datasources_ptr);
-      gnome_config_sync();
     }
   gnome_config_pop_prefix();
   path = g_string_new("");
@@ -704,17 +821,19 @@ gdamgr_store_gda_info(void)
     {
       entry = datasources_ptr->data;
       g_string_sprintf(path, "/gdalib/%s/", entry->gda_name);
-
+      entry->edited = 0;
       gnome_config_push_prefix(path->str);
       gnome_config_set_string("Provider", entry->provider->server_info->name);
       gnome_config_set_string("DSN", entry->dsn);
       gnome_config_set_string("Description", entry->description);
       gnome_config_set_string("Configurator", entry->configurator);
-      gnome_config_sync();
       gnome_config_pop_prefix();
       datasources_ptr = g_list_next(datasources_ptr);
     }
   g_list_free(datasources);
   g_string_free(path, 1);
+  gdamgr_refresh_provider_tree(current_tree);
+  need_save = 0;
+  gnome_config_sync();
   return 0;
 }
