@@ -116,7 +116,7 @@ create_gda_server_config()
   gtk_table_attach(GTK_TABLE(table), entry,
 		   1, 2, 4, 5,
 		   GTK_FILL, 0, 0, 0);
-  save = gtk_button_new_with_label(_("Accept"));
+  save = gtk_button_new_with_label(_("OK"));
   gtk_table_attach(GTK_TABLE(table), save,
 		   0, 2, 5, 6,
 		   GTK_FILL, 0, 0, 0);
@@ -143,7 +143,9 @@ create_gda_dsn_config()
   
   label = gtk_label_new(_("Provider"));
   entry = gtk_entry_new();
-  gtk_entry_set_text(GTK_ENTRY(entry), "Provider");
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+		     GTK_SIGNAL_FUNC(entry_edited), frame);
+  
   gtk_object_set_data(GTK_OBJECT(frame), "provider", entry);
 
   gtk_table_attach(GTK_TABLE(table), label,
@@ -155,8 +157,9 @@ create_gda_dsn_config()
   
   label = gtk_label_new(_("Data Source Name(DSN)"));
   entry = gtk_entry_new();
-  gtk_entry_set_text(GTK_ENTRY(entry), "Data Source Name (DSN)");
   gtk_object_set_data(GTK_OBJECT(frame), "DSN", entry);
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+		     GTK_SIGNAL_FUNC(entry_edited), frame);
 		      
   gtk_table_attach(GTK_TABLE(table), label,
 		   0, 1, 1, 2,
@@ -169,8 +172,9 @@ create_gda_dsn_config()
 
   label = gtk_label_new(_("Description"));
   entry = gtk_entry_new();
-  gtk_entry_set_text(GTK_ENTRY(entry), "Description");
   gtk_object_set_data(GTK_OBJECT(frame), "description", entry);
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+		     GTK_SIGNAL_FUNC(entry_edited), frame);
   gtk_table_attach(GTK_TABLE(table), label,
 		   0, 1, 2, 3,
 		   GTK_FILL, 0, 0, 0);
@@ -180,9 +184,10 @@ create_gda_dsn_config()
 
   label = gtk_label_new(_("Driver Configuration"));
   entry = gtk_entry_new();
-  gtk_entry_set_text(GTK_ENTRY(entry), "Configuration");
   
   gtk_object_set_data(GTK_OBJECT(frame), "configuration", entry);
+  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+		     GTK_SIGNAL_FUNC(entry_edited), frame);
   gtk_table_attach(GTK_TABLE(table), label,
 		   0, 1, 3, 4,
 		   GTK_FILL, 0, 0, 0);
@@ -281,12 +286,13 @@ main(int argc, char* argv[])
   gtk_signal_connect(GTK_OBJECT(app), "delete_event",
 		     GTK_SIGNAL_FUNC(quit_cmd), NULL);
 
+  need_save = 0;
   gdamgr_build_gda_info();
   
   win = create_gda_servers();
   workarea = gtk_notebook_new();
+  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(workarea), FALSE);
   gtk_notebook_set_show_border(GTK_NOTEBOOK(workarea), 0);
-  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(workarea), 1);
   gtk_container_set_border_width(GTK_CONTAINER(workarea), 0);
 
   gtk_object_set_data(GTK_OBJECT(gtk_object_get_data(GTK_OBJECT(win), "ctree")),
