@@ -142,7 +142,7 @@ gda_mysql_cmd_execute        (Gda_MYSQL_Command* cmd, const GDA_CmdParameterSeq*
       return 0;
     }
   recset = gda_mysql_recset_new();
-  recset->mysql = cmd->cnc->mysql;
+  recset->cnc = cmd->cnc;
   recset->mysql_res = mysql_store_result(cmd->cnc->mysql);
   return recset;
 }
