@@ -35,8 +35,12 @@ typedef struct {
    CORBA_boolean attr_prepared;
 
    CORBA_long attr_state;
-  Gda_MYSQL_Command* cmd;
-  
+
+   CORBA_char *attr_text;
+
+   CORBA_unsigned_long attr_type;
+
+   Gda_MYSQL_Command* cmd;
 } impl_POA_GDA_Command;
 
 typedef struct {
@@ -49,8 +53,6 @@ typedef struct {
 
    CORBA_long attr_connectTimeout;
 
-   CORBA_char *attr_dsn;
-
    GDA_CursorLocation attr_cursor;
 
    CORBA_char *attr_version;
@@ -98,26 +100,25 @@ typedef struct {
 static void impl_GNOME_GenericFactory__destroy(impl_POA_GNOME_GenericFactory * servant,
 					       CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GNOME_GenericFactory_supports(impl_POA_GNOME_GenericFactory * servant,
-				   CORBA_char * obj_goad_id,
-				   CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GNOME_GenericFactory_supports(impl_POA_GNOME_GenericFactory * servant,
+				    CORBA_char * obj_goad_id,
+				    CORBA_Environment * ev);
 
-CORBA_Object
-impl_GNOME_GenericFactory_create_object(impl_POA_GNOME_GenericFactory * servant,
-					CORBA_char * goad_id,
-					GNOME_stringlist * params,
-					CORBA_Environment * ev);
+static CORBA_Object
+ impl_GNOME_GenericFactory_create_object(impl_POA_GNOME_GenericFactory * servant,
+					 CORBA_char * goad_id,
+					 GNOME_stringlist * params,
+					 CORBA_Environment * ev);
 
 static void impl_GDA_Fieldx__destroy(impl_POA_GDA_Fieldx * servant,
 				     CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Fieldx_appendChunk(impl_POA_GDA_Fieldx * servant,
+			     GDA_VarBinString * data,
+			     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Fieldx_appendChunk(impl_POA_GDA_Fieldx * servant,
-			    GDA_VarBinString * data,
-			    CORBA_Environment * ev);
-
-GDA_VarBinString *
+static GDA_VarBinString *
  impl_GDA_Fieldx_getChunk(impl_POA_GDA_Fieldx * servant,
 			  CORBA_long amount,
 			  CORBA_Environment * ev);
@@ -125,151 +126,151 @@ GDA_VarBinString *
 static void impl_GDA_Parameter__destroy(impl_POA_GDA_Parameter * servant,
 					CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Parameter_appendChunk(impl_POA_GDA_Parameter * servant,
-			       GDA_Parameter_VarBinString * data,
-			       CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Parameter_appendChunk(impl_POA_GDA_Parameter * servant,
+				GDA_Parameter_VarBinString * data,
+				CORBA_Environment * ev);
 
 static void impl_GDA_Command__destroy(impl_POA_GDA_Command * servant,
 				      CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Command__get_cmdTimeout(impl_POA_GDA_Command * servant,
-				 CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Command__get_cmdTimeout(impl_POA_GDA_Command * servant,
+				  CORBA_Environment * ev);
+static void
  impl_GDA_Command__set_cmdTimeout(impl_POA_GDA_Command * servant,
 				  CORBA_long value,
 				  CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GDA_Command__get_prepared(impl_POA_GDA_Command * servant,
-			       CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_Command__get_prepared(impl_POA_GDA_Command * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Command__get_state(impl_POA_GDA_Command * servant,
-			    CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Command__get_state(impl_POA_GDA_Command * servant,
+			     CORBA_Environment * ev);
+static void
  impl_GDA_Command__set_state(impl_POA_GDA_Command * servant,
 			     CORBA_long value,
 			     CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Command_execute(impl_POA_GDA_Command * servant,
-			 CORBA_char * cmd,
-			 CORBA_long * affected,
-			 GDA_CmdParameterSeq * params,
-			 CORBA_long options,
-			 CORBA_Environment * ev);
+static CORBA_char *
+ impl_GDA_Command__get_text(impl_POA_GDA_Command * servant,
+			    CORBA_Environment * ev);
+static void
+ impl_GDA_Command__set_text(impl_POA_GDA_Command * servant,
+			    CORBA_char * value,
+			    CORBA_Environment * ev);
+
+static CORBA_unsigned_long
+ impl_GDA_Command__get_type(impl_POA_GDA_Command * servant,
+			    CORBA_Environment * ev);
+static void
+ impl_GDA_Command__set_type(impl_POA_GDA_Command * servant,
+			    CORBA_unsigned_long value,
+			    CORBA_Environment * ev);
+
+static GDA_Recordset
+ impl_GDA_Command_open(impl_POA_GDA_Command * servant,
+		       GDA_CmdParameterSeq * param,
+		       GDA_CursorType ct,
+		       GDA_LockType lt,
+		       CORBA_unsigned_long * affected,
+		       CORBA_Environment * ev);
 
 static void impl_GDA_Connection__destroy(impl_POA_GDA_Connection * servant,
 					 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_flags(impl_POA_GDA_Connection * servant,
-			       CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_flags(impl_POA_GDA_Connection * servant,
+				CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_flags(impl_POA_GDA_Connection * servant,
 				CORBA_long value,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_cmdTimeout(impl_POA_GDA_Connection * servant,
-				    CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_cmdTimeout(impl_POA_GDA_Connection * servant,
+				     CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_cmdTimeout(impl_POA_GDA_Connection * servant,
 				     CORBA_long value,
 				     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_connectTimeout(impl_POA_GDA_Connection * servant,
-					CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_connectTimeout(impl_POA_GDA_Connection * servant,
+					 CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_connectTimeout(impl_POA_GDA_Connection * servant,
 					 CORBA_long value,
 					 CORBA_Environment * ev);
 
-GDA_CursorLocation
-impl_GDA_Connection__get_cursor(impl_POA_GDA_Connection * servant,
-				CORBA_Environment * ev);
-void
+static GDA_CursorLocation
+ impl_GDA_Connection__get_cursor(impl_POA_GDA_Connection * servant,
+				 CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_cursor(impl_POA_GDA_Connection * servant,
 				 GDA_CursorLocation value,
 				 CORBA_Environment * ev);
 
-CORBA_char *
+static CORBA_char *
  impl_GDA_Connection__get_version(impl_POA_GDA_Connection * servant,
 				  CORBA_Environment * ev);
 
-GDA_ErrorSeq *
+static GDA_ErrorSeq *
  impl_GDA_Connection__get_errors(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_beginTransaction(impl_POA_GDA_Connection * servant,
-				     CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Connection_commitTransaction(impl_POA_GDA_Connection * servant,
+static CORBA_long
+ impl_GDA_Connection_beginTransaction(impl_POA_GDA_Connection * servant,
 				      CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_rollbackTransaction(impl_POA_GDA_Connection * servant,
-					CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Connection_commitTransaction(impl_POA_GDA_Connection * servant,
+				       CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
+static CORBA_long
+ impl_GDA_Connection_rollbackTransaction(impl_POA_GDA_Connection * servant,
+					 CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
+			   CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
+			  CORBA_char * dsn,
+			  CORBA_char * user,
+			  CORBA_char * passwd,
 			  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_execute(impl_POA_GDA_Connection * servant,
-			    CORBA_char * command,
-			    CORBA_long rowcount,
-			    CORBA_long stmt_type,
-			    CORBA_Environment * ev);
+static GDA_Recordset
+ impl_GDA_Connection_openSchema(impl_POA_GDA_Connection * servant,
+				GDA_Connection_QType t,
+				GDA_Connection_ConstraintSeq * constraints,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
-			 CORBA_char * dsn,
-			 CORBA_char * user,
-			 CORBA_char * passwd,
-			 CORBA_Environment * ev);
+static GDA_Command
+ impl_GDA_Connection_createCommand(impl_POA_GDA_Connection * servant,
+				   CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Connection_openSchema(impl_POA_GDA_Connection * servant,
-			       GDA_Connection_QType t,
-			       GDA_Connection_ConstraintSeq * constraints,
-			       CORBA_Environment * ev);
+static GDA_Recordset
+ impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
+				     CORBA_Environment * ev);
 
-GDA_Command
-impl_GDA_Connection_createCommand(impl_POA_GDA_Connection * servant,
-				  CORBA_Environment * ev);
-
-GDA_Recordset
-impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
-				    CORBA_Environment * ev);
-
-GDA_Connection_DSNlist *
+static GDA_Connection_DSNlist *
  impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Connection_openRecordset(impl_POA_GDA_Connection * servant,
-				  CORBA_char * command,
-				  GDA_LockType lock_type,
-				  GDA_CursorType cursor_type,
-				  CORBA_unsigned_long options,
+static CORBA_long
+ impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
+				  CORBA_char * filename,
 				  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
-				 CORBA_char * filename,
+static CORBA_long
+ impl_GDA_Connection_stopLogging(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_stopLogging(impl_POA_GDA_Connection * servant,
-				CORBA_Environment * ev);
-
-CORBA_char *
+static CORBA_char *
  impl_GDA_Connection_createTable(impl_POA_GDA_Connection * servant,
 				 CORBA_char * name,
 				 GDA_RowAttributes * columns,
@@ -278,155 +279,146 @@ CORBA_char *
 static void impl_GDA_Recordset__destroy(impl_POA_GDA_Recordset * servant,
 					CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_currentBookmark(impl_POA_GDA_Recordset * servant,
-					CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_currentBookmark(impl_POA_GDA_Recordset * servant,
+					 CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_currentBookmark(impl_POA_GDA_Recordset * servant,
 					 CORBA_long value,
 					 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_cachesize(impl_POA_GDA_Recordset * servant,
-				  CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_cachesize(impl_POA_GDA_Recordset * servant,
+				   CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_cachesize(impl_POA_GDA_Recordset * servant,
 				   CORBA_long value,
 				   CORBA_Environment * ev);
 
-GDA_CursorType
-impl_GDA_Recordset__get_currentCursorType(impl_POA_GDA_Recordset * servant,
-					  CORBA_Environment * ev);
-void
+static GDA_CursorType
+ impl_GDA_Recordset__get_currentCursorType(impl_POA_GDA_Recordset * servant,
+					   CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_currentCursorType(impl_POA_GDA_Recordset * servant,
 					   GDA_CursorType value,
 					   CORBA_Environment * ev);
 
-GDA_LockType
-impl_GDA_Recordset__get_lockingMode(impl_POA_GDA_Recordset * servant,
-				    CORBA_Environment * ev);
-void
+static GDA_LockType
+ impl_GDA_Recordset__get_lockingMode(impl_POA_GDA_Recordset * servant,
+				     CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_lockingMode(impl_POA_GDA_Recordset * servant,
 				     GDA_LockType value,
 				     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_maxrecords(impl_POA_GDA_Recordset * servant,
-				   CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_maxrecords(impl_POA_GDA_Recordset * servant,
+				    CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_maxrecords(impl_POA_GDA_Recordset * servant,
 				    CORBA_long value,
 				    CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_pagecount(impl_POA_GDA_Recordset * servant,
-				  CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset__get_pagecount(impl_POA_GDA_Recordset * servant,
+				   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_pagesize(impl_POA_GDA_Recordset * servant,
-				 CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_pagesize(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_pagesize(impl_POA_GDA_Recordset * servant,
 				  CORBA_long value,
 				  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_recCount(impl_POA_GDA_Recordset * servant,
-				 CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset__get_recCount(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
 
-CORBA_char *
+static CORBA_char *
  impl_GDA_Recordset__get_source(impl_POA_GDA_Recordset * servant,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_status(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_cancelBatch(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_cancelUpdate(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset__get_status(impl_POA_GDA_Recordset * servant,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_close(impl_POA_GDA_Recordset * servant,
-			 CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_cancelBatch(impl_POA_GDA_Recordset * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_deleteCurrent(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_cancelUpdate(impl_POA_GDA_Recordset * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_move(impl_POA_GDA_Recordset * servant,
-			CORBA_long count,
-			CORBA_long bookmark,
-			CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_close(impl_POA_GDA_Recordset * servant,
+			  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_moveFirst(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_deleteCurrent(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_move(impl_POA_GDA_Recordset * servant,
+			 CORBA_long count,
+			 CORBA_long bookmark,
+			 CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_moveFirst(impl_POA_GDA_Recordset * servant,
+			      CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_moveLast(impl_POA_GDA_Recordset * servant,
 			     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_moveLast(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_reQuery(impl_POA_GDA_Recordset * servant,
 			    CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_open(impl_POA_GDA_Recordset * servant,
-			CORBA_char * cmd,
-			GDA_Connection cnc,
-			GDA_CursorType ct,
-			GDA_LockType lt,
-			CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_reQuery(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_reSync(impl_POA_GDA_Recordset * servant,
 			   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_reSync(impl_POA_GDA_Recordset * servant,
-			  CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_Recordset_supports(impl_POA_GDA_Recordset * servant,
+			     GDA_Option what,
+			     CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GDA_Recordset_supports(impl_POA_GDA_Recordset * servant,
-			    GDA_Option what,
-			    CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_update(impl_POA_GDA_Recordset * servant,
+			   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_update(impl_POA_GDA_Recordset * servant,
-			  CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_updateBatch(impl_POA_GDA_Recordset * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_updateBatch(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-GDA_Recordset_Chunk *
+static GDA_Recordset_Chunk *
  impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
 			  CORBA_long count,
 			  CORBA_Environment * ev);
 
-GDA_RowAttributes *
+static GDA_RowAttributes *
  impl_GDA_Recordset_describe(impl_POA_GDA_Recordset * servant,
 			     CORBA_Environment * ev);
 
 static void impl_GDA_ConnectionFactory__destroy(impl_POA_GDA_ConnectionFactory * servant,
 						CORBA_Environment * ev);
+static CORBA_Object
+ impl_GDA_ConnectionFactory_create_connection(impl_POA_GDA_ConnectionFactory * servant,
+					      CORBA_char * goad_id,
+					      CORBA_Environment * ev);
 
-CORBA_Object
-impl_GDA_ConnectionFactory_create_connection(impl_POA_GDA_ConnectionFactory * servant,
-					     CORBA_char * goad_id,
-					     CORBA_Environment * ev);
-
-CORBA_boolean
-impl_GDA_ConnectionFactory_supports(impl_POA_GDA_ConnectionFactory * servant,
-				    CORBA_char * obj_goad_id,
-				    CORBA_Environment * ev);
-CORBA_Object
-impl_GDA_ConnectionFactory_create_object(impl_POA_GDA_ConnectionFactory * servant,
-					 CORBA_char * goad_id,
-					 GNOME_stringlist * params,
-					 CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_ConnectionFactory_supports(impl_POA_GDA_ConnectionFactory * servant,
+				     CORBA_char * obj_goad_id,
+				     CORBA_Environment * ev);
+static CORBA_Object
+ impl_GDA_ConnectionFactory_create_object(impl_POA_GDA_ConnectionFactory * servant,
+					  CORBA_char * goad_id,
+					  GNOME_stringlist * params,
+					  CORBA_Environment * ev);
 
 /*** epv structures ***/
 
@@ -492,7 +484,13 @@ static POA_GDA_Command__epv impl_GDA_Command_epv =
    (gpointer) & impl_GDA_Command__get_state,
    (gpointer) & impl_GDA_Command__set_state,
 
-   (gpointer) & impl_GDA_Command_execute,
+   (gpointer) & impl_GDA_Command__get_text,
+   (gpointer) & impl_GDA_Command__set_text,
+
+   (gpointer) & impl_GDA_Command__get_type,
+   (gpointer) & impl_GDA_Command__set_type,
+
+   (gpointer) & impl_GDA_Command_open,
 
 };
 static PortableServer_ServantBase__epv impl_GDA_Connection_base_epv =
@@ -529,8 +527,6 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
 
    (gpointer) & impl_GDA_Connection_close,
 
-   (gpointer) & impl_GDA_Connection_execute,
-
    (gpointer) & impl_GDA_Connection_open,
 
    (gpointer) & impl_GDA_Connection_openSchema,
@@ -541,8 +537,6 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
 
    (gpointer) & impl_GDA_Connection_listSources,
 
-   (gpointer) & impl_GDA_Connection_openRecordset,
-
    (gpointer) & impl_GDA_Connection_startLogging,
 
    (gpointer) & impl_GDA_Connection_stopLogging,
@@ -601,8 +595,6 @@ static POA_GDA_Recordset__epv impl_GDA_Recordset_epv =
 
    (gpointer) & impl_GDA_Recordset_moveLast,
 
-   (gpointer) & impl_GDA_Recordset_open,
-
    (gpointer) & impl_GDA_Recordset_reQuery,
 
    (gpointer) & impl_GDA_Recordset_reSync,
@@ -944,25 +936,62 @@ impl_GDA_Command__set_state(impl_POA_GDA_Command * servant,
 {
    g_error("%s not implemented", __PRETTY_FUNCTION__);
 }
+static CORBA_char *
+impl_GDA_Command__get_text(impl_POA_GDA_Command * servant,
+			   CORBA_Environment * ev)
+{
+
+   CORBA_char *retval;
+   if (servant->cmd->cmd)
+     retval = CORBA_string_dup(servant->cmd->cmd);
+   else
+     retval = 0;
+   return retval;
+}
+
+static void
+impl_GDA_Command__set_text(impl_POA_GDA_Command * servant,
+			   CORBA_char * value,
+			   CORBA_Environment * ev)
+{
+  if (servant->cmd->cmd)
+    {
+      g_free(servant->cmd->cmd);
+    }
+  servant->cmd->cmd = g_strdup(value);
+  return;
+}
+
+static CORBA_unsigned_long
+impl_GDA_Command__get_type(impl_POA_GDA_Command * servant,
+			   CORBA_Environment * ev)
+{
+   CORBA_unsigned_long retval;
+   retval = servant->cmd->type;
+   return retval;
+}
+
+static void
+impl_GDA_Command__set_type(impl_POA_GDA_Command * servant,
+			   CORBA_unsigned_long value,
+			   CORBA_Environment * ev)
+{
+  servant->cmd->type = value;
+}
+
 
 GDA_Recordset
-impl_GDA_Command_execute(impl_POA_GDA_Command * servant,
-			 CORBA_char * cmd,
-			 CORBA_long * affected,
-			 GDA_CmdParameterSeq * params,
-			 CORBA_long options,
-			 CORBA_Environment * ev)
+impl_GDA_Command_open(impl_POA_GDA_Command * servant,
+		      GDA_CmdParameterSeq * param,
+		       GDA_CursorType ct,
+		       GDA_LockType lt,
+		       CORBA_unsigned_long * affected,
+		       CORBA_Environment * ev)
 {
   GDA_Recordset             retval = 0;
   Gda_MYSQL_Recordset*      recset;
   
-  fprintf(stderr,"gda_mysql_Command_execute: cmd = '%s'\n", cmd);
-  if (servant->cmd->cmd) {
-    g_warning("WARNING WARNING: command string still existing");
-    g_free(servant->cmd->cmd);
-  }
-  servant->cmd->cmd = g_strdup(cmd);
-  recset = gda_mysql_cmd_execute(servant->cmd, params, 0);
+  recset = gda_mysql_cmd_execute(servant->cmd, param, 0);
   if (!recset)
     {
       GDA_DriverError* exception = GDA_DriverError__alloc();
@@ -1195,19 +1224,6 @@ impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
   
 }
 
-CORBA_long
-impl_GDA_Connection_execute(impl_POA_GDA_Connection * servant,
-			    CORBA_char * command,
-			    CORBA_long rowcount,
-			    CORBA_long stmt_type,
-			    CORBA_Environment * ev)
-{
-   CORBA_long retval = -1;
-
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
-   return retval;
-}
-
 CORBA_long
 impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
 			 CORBA_char * dsn,
@@ -1269,9 +1285,15 @@ GDA_Recordset
 impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
 				    CORBA_Environment * ev)
 {
-   GDA_Recordset retval;
+  GDA_Recordset retval = 0;
+  Gda_MYSQL_Recordset* rs;
 
-   return retval;
+  rs = gda_mysql_recset_new();
+  rs->cnc = servant->cnc;
+  retval = impl_GDA_Recordset__create(servant->poa, rs, ev);
+  if (Exception(ev))
+    return CORBA_OBJECT_NIL;
+  return retval;
 }
 
 GDA_Connection_DSNlist *
@@ -1283,48 +1305,6 @@ impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
   return retval;
 }
 
-GDA_Recordset
-impl_GDA_Connection_openRecordset(impl_POA_GDA_Connection * servant,
-				  CORBA_char * command,
-				  GDA_LockType lock_type,
-				  GDA_CursorType cursor_type,
-				  CORBA_unsigned_long options,
-				  CORBA_Environment * ev)
-{
-  GDA_Recordset            new_recset;
-  Gda_MYSQL_Recordset*      recset;
-  Gda_MYSQL_Command*        cmd;
-
-  cmd = gda_mysql_cmd_new();
-  cmd->cmd = g_strdup(command);
-  cmd->cnc  = servant->cnc;
-  recset = gda_mysql_cmd_execute(cmd, 0, 0);
-  if (!recset)
-    {
-      GDA_DriverError*     exception = GDA_DriverError__alloc();
-      Gda_MYSQL_Connection* mysql_cnc = servant->cnc;
-#if 0
-      exception->errors._length = 1;
-      exception->errors._buffer = CORBA_sequence_GDA_Error_allocbuf(1);
-      exception->errors._buffer[0].description = CORBA_string_dup("MySQL Error");
-      exception->errors._buffer[0].number      = -1;
-      exception->errors._buffer[0].source      = CORBA_string_dup("[MySQL]");
-      exception->errors._buffer[0].sqlstate    = CORBA_string_dup("MySQL Error State");
-      exception->errors._buffer[0].nativeMsg   = CORBA_string_dup("MySQL Error Native");
-      exception->realcommand = CORBA_string_dup("<Unknown>");
-      g_warning("MySQL: Command execute sends a CORBA exception");
-#else      
-      exception->errors._length = g_list_length(mysql_cnc->errors);
-      exception->errors._buffer = make_errorbuffer(mysql_cnc);
-      exception->realcommand = CORBA_string_dup((CORBA_char*)command);
-#endif
-      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
-      return CORBA_OBJECT_NIL;
-    }
-  new_recset = impl_GDA_Recordset__create(servant->poa, recset, ev);
-  return new_recset;
-}
-
 CORBA_long
 impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
 				 CORBA_char * filename,
@@ -1352,9 +1332,9 @@ impl_GDA_Connection_createTable(impl_POA_GDA_Connection * servant,
 				GDA_RowAttributes * columns,
 				CORBA_Environment * ev)
 {
-   CORBA_char *retval;
-
-   return retval;
+  CORBA_char *retval = 0;
+  g_error("%s not implemented", __PRETTY_FUNCTION__);
+  return retval;
 }
 
 static GDA_Recordset 
@@ -1623,18 +1603,45 @@ impl_GDA_Recordset_moveNext(impl_POA_GDA_Recordset * servant,
 }
 
 
-CORBA_long
-impl_GDA_Recordset_open(impl_POA_GDA_Recordset * servant,
-			CORBA_char * cmd,
-			GDA_Connection cnc,
-			GDA_CursorType ct,
-			GDA_LockType lt,
-			CORBA_Environment * ev)
+static CORBA_long
+ impl_GDA_Recordset_open(impl_POA_GDA_Recordset * servant,
+			 CORBA_char * text,
+			 CORBA_unsigned_long type,
+			 GDA_CmdParameterSeq * param,
+			 GDA_CursorType ct,
+			 GDA_LockType lt,
+			 CORBA_long * affected,
+			 CORBA_Environment * ev)
 {
-   CORBA_long retval = 0;
+  Gda_MYSQL_Command*        gda_cmd;
+  
+  *affected = 0;
+  CORBA_exception_init(ev);
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
-   return retval;
+  gda_cmd = gda_mysql_cmd_new();
+  gda_cmd->cnc = servant->rs->cnc;
+  gda_cmd->cmd = g_strdup(text);
+  gda_cmd->type = type;
+
+  servant->rs = gda_mysql_cmd_execute(gda_cmd, param, 0);
+  if (!servant->rs)
+    {
+      GDA_DriverError* exception = GDA_DriverError__alloc();
+      exception->errors._length = 1;
+      exception->errors._buffer = CORBA_sequence_GDA_Error_allocbuf(1);
+      exception->errors._buffer[0].description = CORBA_string_dup("MySQL Error");
+      exception->errors._buffer[0].number      = -1;
+      exception->errors._buffer[0].source      = CORBA_string_dup("[MySQL]");
+      exception->errors._buffer[0].sqlstate    = CORBA_string_dup("MySQL Error State");
+      exception->errors._buffer[0].nativeMsg   = CORBA_string_dup("MySQL Error Native");
+      exception->realcommand = CORBA_string_dup("<Unknown>");
+      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
+      g_warning("MySQL: Command execute sends a CORBA exception");
+      return -1;
+    }
+  if (!servant->rs)
+    return -1;
+  return 0;
 }
 
 CORBA_long
@@ -1717,7 +1724,7 @@ impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
       if (!mysql_row)
 	{
 	  fprintf(stderr,"mysql_fetch_row returns NULL\n");
-	  fprintf(stderr,"error '%s'\n", mysql_error(rs->mysql));
+	  fprintf(stderr,"error '%s'\n", mysql_error(rs->cnc->mysql));
 	  break;
 	}
       if (rs->fix_fields)
