@@ -99,11 +99,8 @@ schema_tables(Gda_ODBC_Recordset*   recset,
 	      gint length)
 {
   gint                rc;
-  Gda_ODBC_Command*   cmd;
   
-  cmd = gda_odbc_cmd_new();
-  cmd->cnc = cnc;
-  recset->cmd = cmd;
+  recset->cnc = cnc;
   rc = SQLAllocStmt(cnc->hdbc, &recset->hstmt);
   fprintf(stderr,"schema_tables: Step1: cnc = %p, cnc->hdbc = %p, recset->hstmt = %p\n", cnc, cnc->hdbc, recset->hstmt);
   if (rc != SQL_SUCCESS)
@@ -149,11 +146,8 @@ schema_views(Gda_ODBC_Recordset*   recset,
 	     gint length)
 {
   gint                rc;
-  Gda_ODBC_Command*   cmd;
   
-  cmd = gda_odbc_cmd_new();
-  cmd->cnc = cnc;
-  recset->cmd = cmd;
+  recset->cnc = cnc;
   rc = SQLAllocStmt(cnc->hdbc, &recset->hstmt);
   if (rc != SQL_SUCCESS)
     {
@@ -197,7 +191,6 @@ schema_procedures(Gda_ODBC_Recordset*   recset,
   
   cmd = gda_odbc_cmd_new();
   cmd->cnc = cnc;
-  recset->cmd = cmd;
   rc = SQLAllocStmt(cnc->hdbc, &recset->hstmt);
   if (rc != SQL_SUCCESS)
     {
@@ -229,16 +222,13 @@ schema_columns(Gda_ODBC_Recordset*  recset,
 	       gint            length)
 {
   gint             rc;
-  Gda_ODBC_Command*     cmd;
   gchar*           table_qualifier = 0;
   gchar*           table_owner     = 0;
   gchar*           table_name      = 0;
   gchar*           column_name     = 0;
   GDA_Connection_Constraint*  ptr;
   
-  cmd = gda_odbc_cmd_new();
-  cmd->cnc = cnc;
-  recset->cmd = cmd;
+  recset->cnc = cnc;
   rc = SQLAllocStmt(cnc->hdbc, &recset->hstmt);
   if (rc != SQL_SUCCESS)
     {
@@ -295,11 +285,8 @@ schema_types(Gda_ODBC_Recordset*   recset,
 	     gint             length)
 {
   gint              rc;
-  Gda_ODBC_Command* cmd;
 
-  cmd = gda_odbc_cmd_new();
-  cmd->cnc = cnc;
-  recset->cmd = cmd;
+  recset->cnc = cnc;
   rc = SQLAllocStmt(cnc->hdbc, &recset->hstmt);
   if (rc == SQL_ERROR)
     return 0;
