@@ -17,6 +17,7 @@
  */
 
 #include <memory.h>
+#include <ctype.h>
 #include "gda-error.h"
 
 Gda_ODBC_Error*
@@ -43,7 +44,32 @@ gda_odbc_error_free(Gda_ODBC_Error* e)
     g_free(e->native);
   g_free(e);
 }
+
+
+static gchar*
+find_end_of_location(gchar* error)
+{
+  gint in_brackets = 0;
   
+  for (;*error; error++)
+    {
+      if (*error == '[')
+	{
+	  in_brackets = 1;
+	  continue;
+	}
+      if (*error == ']')
+	{
+	  in_brackets = 0;
+	  continue;
+	}
+      if (!isspace(*error) && !in_brackets)
+	return error;
+    }
+  return 0;
+}
+      
+
 void
 gda_odbc_error_make(Gda_ODBC_Recordset*  rec,
 		    Gda_ODBC_Connection* cnc,
@@ -54,6 +80,9 @@ gda_odbc_error_make(Gda_ODBC_Recordset*  rec,
   HSTMT hstmt;
   gint  rc;
   Gda_ODBC_Error* e;
+
+  gchar* eol;
+  gchar* bol;
   
   static UCHAR  sqlstate[SQL_MAX_MESSAGE_LENGTH];
   static UCHAR  errMsg[SQL_MAX_MESSAGE_LENGTH];
@@ -62,10 +91,10 @@ gda_odbc_error_make(Gda_ODBC_Recordset*  rec,
   
   if (rec)
     {
-      cnc = rec->cmd->cnc;
+      cnc = rec->cnc;
       hstmt = rec->hstmt;
-      henv  = rec->cmd->cnc->henv;
-      hdbc  = rec->cmd->cnc->hdbc;
+      henv  = rec->cnc->henv;
+      hdbc  = rec->cnc->hdbc;
     }
   else if (cnc)
     {
@@ -82,15 +111,20 @@ gda_odbc_error_make(Gda_ODBC_Recordset*  rec,
   rc = SQLError(henv, hdbc, hstmt, sqlstate, &e->number, errMsg,
 		sizeof(errMsg)-1, &actLength);
 
+
   fprintf(stderr,"Error at '%s': ", where);
   fprintf(stderr,"error->description = '%s'\n", errMsg);
+  fprintf(stderr,"   end of location = '%s'\n", find_end_of_location(errMsg));
   fprintf(stderr,"   sqlstate = '%s'\n", sqlstate);
-  e->description = g_strdup(errMsg);
-  e->source      = g_strdup("gda-odbc");
-  e->helpfile    = 0;
-  e->helpctxt    = 0;
+
+  bol = errMsg;
+  e->description = g_strdup(eol = find_end_of_location(errMsg));
+  e->source      = g_strdup_printf("[gda-odbc]%0.*s", (eol - bol), errMsg);
+
+  e->helpfile    = g_strdup("error");
+  e->helpctxt    = g_strdup("error");
   e->sqlstate    = g_strdup(sqlstate);
-  e->native      = 0;
+  e->native      = g_strdup(eol);
   cnc->errors = g_list_append(cnc->errors, e);
 }
 
