@@ -8,8 +8,7 @@
 
 Gda_ODBC_Error*  gda_odbc_error_new   (void);
 void             gda_odbc_error_free  (Gda_ODBC_Error* e);
-void             gda_odbc_error_make  (Gda_ODBC_Error* e,
-				       Gda_ODBC_Recordset*  rec,
+void             gda_odbc_error_make  (Gda_ODBC_Recordset*  rec,
 				       Gda_ODBC_Connection* cnc,
 				       gchar*          where);
 						     
