@@ -96,26 +96,25 @@ typedef struct {
 static void impl_GNOME_GenericFactory__destroy(impl_POA_GNOME_GenericFactory * servant,
 					       CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GNOME_GenericFactory_supports(impl_POA_GNOME_GenericFactory * servant,
-				   CORBA_char * obj_goad_id,
-				   CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GNOME_GenericFactory_supports(impl_POA_GNOME_GenericFactory * servant,
+				    CORBA_char * obj_goad_id,
+				    CORBA_Environment * ev);
 
-CORBA_Object
-impl_GNOME_GenericFactory_create_object(impl_POA_GNOME_GenericFactory * servant,
-					CORBA_char * goad_id,
-					GNOME_stringlist * params,
-					CORBA_Environment * ev);
+static CORBA_Object
+ impl_GNOME_GenericFactory_create_object(impl_POA_GNOME_GenericFactory * servant,
+					 CORBA_char * goad_id,
+					 GNOME_stringlist * params,
+					 CORBA_Environment * ev);
 
 static void impl_GDA_Fieldx__destroy(impl_POA_GDA_Fieldx * servant,
 				     CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Fieldx_appendChunk(impl_POA_GDA_Fieldx * servant,
+			     GDA_VarBinString * data,
+			     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Fieldx_appendChunk(impl_POA_GDA_Fieldx * servant,
-			    GDA_VarBinString * data,
-			    CORBA_Environment * ev);
-
-GDA_VarBinString *
+static GDA_VarBinString *
  impl_GDA_Fieldx_getChunk(impl_POA_GDA_Fieldx * servant,
 			  CORBA_long amount,
 			  CORBA_Environment * ev);
@@ -123,151 +122,151 @@ GDA_VarBinString *
 static void impl_GDA_Parameter__destroy(impl_POA_GDA_Parameter * servant,
 					CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Parameter_appendChunk(impl_POA_GDA_Parameter * servant,
-			       GDA_Parameter_VarBinString * data,
-			       CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Parameter_appendChunk(impl_POA_GDA_Parameter * servant,
+				GDA_Parameter_VarBinString * data,
+				CORBA_Environment * ev);
 
 static void impl_GDA_Command__destroy(impl_POA_GDA_Command * servant,
 				      CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Command__get_cmdTimeout(impl_POA_GDA_Command * servant,
-				 CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Command__get_cmdTimeout(impl_POA_GDA_Command * servant,
+				  CORBA_Environment * ev);
+static void
  impl_GDA_Command__set_cmdTimeout(impl_POA_GDA_Command * servant,
 				  CORBA_long value,
 				  CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GDA_Command__get_prepared(impl_POA_GDA_Command * servant,
-			       CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_Command__get_prepared(impl_POA_GDA_Command * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Command__get_state(impl_POA_GDA_Command * servant,
-			    CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Command__get_state(impl_POA_GDA_Command * servant,
+			     CORBA_Environment * ev);
+static void
  impl_GDA_Command__set_state(impl_POA_GDA_Command * servant,
 			     CORBA_long value,
 			     CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Command_execute(impl_POA_GDA_Command * servant,
-			 CORBA_char * cmd,
-			 CORBA_long * affected,
-			 GDA_CmdParameterSeq * params,
-			 CORBA_long options,
-			 CORBA_Environment * ev);
+static CORBA_char *
+ impl_GDA_Command__get_text(impl_POA_GDA_Command * servant,
+			    CORBA_Environment * ev);
+static void
+ impl_GDA_Command__set_text(impl_POA_GDA_Command * servant,
+			    CORBA_char * value,
+			    CORBA_Environment * ev);
+
+static CORBA_unsigned_long
+ impl_GDA_Command__get_type(impl_POA_GDA_Command * servant,
+			    CORBA_Environment * ev);
+static void
+ impl_GDA_Command__set_type(impl_POA_GDA_Command * servant,
+			    CORBA_unsigned_long value,
+			    CORBA_Environment * ev);
+
+static GDA_Recordset
+ impl_GDA_Command_open(impl_POA_GDA_Command * servant,
+		       GDA_CmdParameterSeq * param,
+		       GDA_CursorType ct,
+		       GDA_LockType lt,
+		       CORBA_unsigned_long * affected,
+		       CORBA_Environment * ev);
 
 static void impl_GDA_Connection__destroy(impl_POA_GDA_Connection * servant,
 					 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_flags(impl_POA_GDA_Connection * servant,
-			       CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_flags(impl_POA_GDA_Connection * servant,
+				CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_flags(impl_POA_GDA_Connection * servant,
 				CORBA_long value,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_cmdTimeout(impl_POA_GDA_Connection * servant,
-				    CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_cmdTimeout(impl_POA_GDA_Connection * servant,
+				     CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_cmdTimeout(impl_POA_GDA_Connection * servant,
 				     CORBA_long value,
 				     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection__get_connectTimeout(impl_POA_GDA_Connection * servant,
-					CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Connection__get_connectTimeout(impl_POA_GDA_Connection * servant,
+					 CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_connectTimeout(impl_POA_GDA_Connection * servant,
 					 CORBA_long value,
 					 CORBA_Environment * ev);
 
-GDA_CursorLocation
-impl_GDA_Connection__get_cursor(impl_POA_GDA_Connection * servant,
-				CORBA_Environment * ev);
-void
+static GDA_CursorLocation
+ impl_GDA_Connection__get_cursor(impl_POA_GDA_Connection * servant,
+				 CORBA_Environment * ev);
+static void
  impl_GDA_Connection__set_cursor(impl_POA_GDA_Connection * servant,
 				 GDA_CursorLocation value,
 				 CORBA_Environment * ev);
 
-CORBA_char *
+static CORBA_char *
  impl_GDA_Connection__get_version(impl_POA_GDA_Connection * servant,
 				  CORBA_Environment * ev);
 
-GDA_ErrorSeq *
+static GDA_ErrorSeq *
  impl_GDA_Connection__get_errors(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_beginTransaction(impl_POA_GDA_Connection * servant,
-				     CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Connection_commitTransaction(impl_POA_GDA_Connection * servant,
+static CORBA_long
+ impl_GDA_Connection_beginTransaction(impl_POA_GDA_Connection * servant,
 				      CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_rollbackTransaction(impl_POA_GDA_Connection * servant,
-					CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Connection_commitTransaction(impl_POA_GDA_Connection * servant,
+				       CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
+static CORBA_long
+ impl_GDA_Connection_rollbackTransaction(impl_POA_GDA_Connection * servant,
+					 CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
+			   CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
+			  CORBA_char * dsn,
+			  CORBA_char * user,
+			  CORBA_char * passwd,
 			  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_execute(impl_POA_GDA_Connection * servant,
-			    CORBA_char * command,
-			    CORBA_long rowcount,
-			    CORBA_long stmt_type,
-			    CORBA_Environment * ev);
+static GDA_Recordset
+ impl_GDA_Connection_openSchema(impl_POA_GDA_Connection * servant,
+				GDA_Connection_QType t,
+				GDA_Connection_ConstraintSeq * constraints,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
-			 CORBA_char * dsn,
-			 CORBA_char * user,
-			 CORBA_char * passwd,
-			 CORBA_Environment * ev);
+static GDA_Command
+ impl_GDA_Connection_createCommand(impl_POA_GDA_Connection * servant,
+				   CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Connection_openSchema(impl_POA_GDA_Connection * servant,
-			       GDA_Connection_QType t,
-			       GDA_Connection_ConstraintSeq * constraints,
-			       CORBA_Environment * ev);
+static GDA_Recordset
+ impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
+				     CORBA_Environment * ev);
 
-GDA_Command
-impl_GDA_Connection_createCommand(impl_POA_GDA_Connection * servant,
-				  CORBA_Environment * ev);
-
-GDA_Recordset
-impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
-				    CORBA_Environment * ev);
-
-GDA_Connection_DSNlist *
+static GDA_Connection_DSNlist *
  impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-GDA_Recordset
-impl_GDA_Connection_openRecordset(impl_POA_GDA_Connection * servant,
-				  CORBA_char * command,
-				  GDA_LockType lock_type,
-				  GDA_CursorType cursor_type,
-				  CORBA_unsigned_long options,
+static CORBA_long
+ impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
+				  CORBA_char * filename,
 				  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
-				 CORBA_char * filename,
+static CORBA_long
+ impl_GDA_Connection_stopLogging(impl_POA_GDA_Connection * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Connection_stopLogging(impl_POA_GDA_Connection * servant,
-				CORBA_Environment * ev);
-
-CORBA_char *
+static CORBA_char *
  impl_GDA_Connection_createTable(impl_POA_GDA_Connection * servant,
 				 CORBA_char * name,
 				 GDA_RowAttributes * columns,
@@ -276,156 +275,146 @@ CORBA_char *
 static void impl_GDA_Recordset__destroy(impl_POA_GDA_Recordset * servant,
 					CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_currentBookmark(impl_POA_GDA_Recordset * servant,
-					CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_currentBookmark(impl_POA_GDA_Recordset * servant,
+					 CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_currentBookmark(impl_POA_GDA_Recordset * servant,
 					 CORBA_long value,
 					 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_cachesize(impl_POA_GDA_Recordset * servant,
-				  CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_cachesize(impl_POA_GDA_Recordset * servant,
+				   CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_cachesize(impl_POA_GDA_Recordset * servant,
 				   CORBA_long value,
 				   CORBA_Environment * ev);
 
-GDA_CursorType
-impl_GDA_Recordset__get_currentCursorType(impl_POA_GDA_Recordset * servant,
-					  CORBA_Environment * ev);
-void
+static GDA_CursorType
+ impl_GDA_Recordset__get_currentCursorType(impl_POA_GDA_Recordset * servant,
+					   CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_currentCursorType(impl_POA_GDA_Recordset * servant,
 					   GDA_CursorType value,
 					   CORBA_Environment * ev);
 
-GDA_LockType
-impl_GDA_Recordset__get_lockingMode(impl_POA_GDA_Recordset * servant,
-				    CORBA_Environment * ev);
-void
+static GDA_LockType
+ impl_GDA_Recordset__get_lockingMode(impl_POA_GDA_Recordset * servant,
+				     CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_lockingMode(impl_POA_GDA_Recordset * servant,
 				     GDA_LockType value,
 				     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_maxrecords(impl_POA_GDA_Recordset * servant,
-				   CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_maxrecords(impl_POA_GDA_Recordset * servant,
+				    CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_maxrecords(impl_POA_GDA_Recordset * servant,
 				    CORBA_long value,
 				    CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_pagecount(impl_POA_GDA_Recordset * servant,
-				  CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset__get_pagecount(impl_POA_GDA_Recordset * servant,
+				   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_pagesize(impl_POA_GDA_Recordset * servant,
-				 CORBA_Environment * ev);
-void
+static CORBA_long
+ impl_GDA_Recordset__get_pagesize(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
+static void
  impl_GDA_Recordset__set_pagesize(impl_POA_GDA_Recordset * servant,
 				  CORBA_long value,
 				  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_recCount(impl_POA_GDA_Recordset * servant,
-				 CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset__get_recCount(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
 
-CORBA_char *
+static CORBA_char *
  impl_GDA_Recordset__get_source(impl_POA_GDA_Recordset * servant,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset__get_status(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_cancelBatch(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_cancelUpdate(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset__get_status(impl_POA_GDA_Recordset * servant,
 				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_close(impl_POA_GDA_Recordset * servant,
-			 CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_cancelBatch(impl_POA_GDA_Recordset * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_deleteCurrent(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_cancelUpdate(impl_POA_GDA_Recordset * servant,
 				 CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_move(impl_POA_GDA_Recordset * servant,
-			CORBA_long count,
-			CORBA_long bookmark,
-			CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_close(impl_POA_GDA_Recordset * servant,
+			  CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_moveFirst(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_deleteCurrent(impl_POA_GDA_Recordset * servant,
+				  CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_move(impl_POA_GDA_Recordset * servant,
+			 CORBA_long count,
+			 CORBA_long bookmark,
+			 CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_moveFirst(impl_POA_GDA_Recordset * servant,
+			      CORBA_Environment * ev);
+
+static CORBA_long
+ impl_GDA_Recordset_moveLast(impl_POA_GDA_Recordset * servant,
 			     CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_moveLast(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_reQuery(impl_POA_GDA_Recordset * servant,
 			    CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_open(impl_POA_GDA_Recordset * servant,
-			CORBA_char * cmd,
-			GDA_Connection cnc,
-			GDA_CursorType ct,
-			GDA_LockType lt,
-			CORBA_Environment * ev);
-
-CORBA_long
-impl_GDA_Recordset_reQuery(impl_POA_GDA_Recordset * servant,
+static CORBA_long
+ impl_GDA_Recordset_reSync(impl_POA_GDA_Recordset * servant,
 			   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_reSync(impl_POA_GDA_Recordset * servant,
-			  CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_Recordset_supports(impl_POA_GDA_Recordset * servant,
+			     GDA_Option what,
+			     CORBA_Environment * ev);
 
-CORBA_boolean
-impl_GDA_Recordset_supports(impl_POA_GDA_Recordset * servant,
-			    GDA_Option what,
-			    CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_update(impl_POA_GDA_Recordset * servant,
+			   CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_update(impl_POA_GDA_Recordset * servant,
-			  CORBA_Environment * ev);
+static CORBA_long
+ impl_GDA_Recordset_updateBatch(impl_POA_GDA_Recordset * servant,
+				CORBA_Environment * ev);
 
-CORBA_long
-impl_GDA_Recordset_updateBatch(impl_POA_GDA_Recordset * servant,
-			       CORBA_Environment * ev);
-
-GDA_Recordset_Chunk *
+static GDA_Recordset_Chunk *
  impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
 			  CORBA_long count,
 			  CORBA_Environment * ev);
 
-GDA_RowAttributes *
+static GDA_RowAttributes *
  impl_GDA_Recordset_describe(impl_POA_GDA_Recordset * servant,
 			     CORBA_Environment * ev);
 
+static void impl_GDA_ConnectionFactory__destroy(impl_POA_GDA_ConnectionFactory * servant,
+						CORBA_Environment * ev);
+static CORBA_Object
+ impl_GDA_ConnectionFactory_create_connection(impl_POA_GDA_ConnectionFactory * servant,
+					      CORBA_char * goad_id,
+					      CORBA_Environment * ev);
 
-void impl_GDA_ConnectionFactory__destroy(impl_POA_GDA_ConnectionFactory * servant,
-					 CORBA_Environment * ev);
-
-CORBA_Object
-impl_GDA_ConnectionFactory_create_connection(impl_POA_GDA_ConnectionFactory * servant,
-					     CORBA_char * goad_id,
-					     CORBA_Environment * ev);
-
-CORBA_boolean
-impl_GDA_ConnectionFactory_supports(impl_POA_GDA_ConnectionFactory * servant,
-				    CORBA_char * obj_goad_id,
-				    CORBA_Environment * ev);
-CORBA_Object
-impl_GDA_ConnectionFactory_create_object(impl_POA_GDA_ConnectionFactory * servant,
-					 CORBA_char * goad_id,
-					 GNOME_stringlist * params,
-					 CORBA_Environment * ev);
+static CORBA_boolean
+ impl_GDA_ConnectionFactory_supports(impl_POA_GDA_ConnectionFactory * servant,
+				     CORBA_char * obj_goad_id,
+				     CORBA_Environment * ev);
+static CORBA_Object
+ impl_GDA_ConnectionFactory_create_object(impl_POA_GDA_ConnectionFactory * servant,
+					  CORBA_char * goad_id,
+					  GNOME_stringlist * params,
+					  CORBA_Environment * ev);
 
 /*** epv structures ***/
 
@@ -491,7 +480,13 @@ static POA_GDA_Command__epv impl_GDA_Command_epv =
    (gpointer) & impl_GDA_Command__get_state,
    (gpointer) & impl_GDA_Command__set_state,
 
-   (gpointer) & impl_GDA_Command_execute,
+   (gpointer) & impl_GDA_Command__get_text,
+   (gpointer) & impl_GDA_Command__set_text,
+
+   (gpointer) & impl_GDA_Command__get_type,
+   (gpointer) & impl_GDA_Command__set_type,
+
+   (gpointer) & impl_GDA_Command_open,
 
 };
 static PortableServer_ServantBase__epv impl_GDA_Connection_base_epv =
@@ -528,8 +523,6 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
 
    (gpointer) & impl_GDA_Connection_close,
 
-   (gpointer) & impl_GDA_Connection_execute,
-
    (gpointer) & impl_GDA_Connection_open,
 
    (gpointer) & impl_GDA_Connection_openSchema,
@@ -540,8 +533,6 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
 
    (gpointer) & impl_GDA_Connection_listSources,
 
-   (gpointer) & impl_GDA_Connection_openRecordset,
-
    (gpointer) & impl_GDA_Connection_startLogging,
 
    (gpointer) & impl_GDA_Connection_stopLogging,
@@ -600,8 +591,6 @@ static POA_GDA_Recordset__epv impl_GDA_Recordset_epv =
 
    (gpointer) & impl_GDA_Recordset_moveLast,
 
-   (gpointer) & impl_GDA_Recordset_open,
-
    (gpointer) & impl_GDA_Recordset_reQuery,
 
    (gpointer) & impl_GDA_Recordset_reSync,
@@ -711,7 +700,7 @@ make_errorbuffer(Gda_ODBC_Connection* cnc)
   return rc;
 }      
 
-int
+static int
 Exception( CORBA_Environment* ev )
 {
   switch( ev->_major )
@@ -884,7 +873,8 @@ impl_GDA_Command__create(PortableServer_POA poa, Gda_ODBC_Command* cmd, CORBA_En
    objid = PortableServer_POA_activate_object(poa, newservant, ev);
    CORBA_free(objid);
    retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);
-
+   g_print("%s: New reference at 0x%p\n", __PRETTY_FUNCTION__, retval);
+   g_print("    stringifyed: '%s'\n", CORBA_ORB_object_to_string(poa->orb, retval, ev));
    return retval;
 }
 
@@ -943,45 +933,82 @@ impl_GDA_Command__set_state(impl_POA_GDA_Command * servant,
    g_error("%s not implemented", __PRETTY_FUNCTION__);
 }
 
-GDA_Recordset
-impl_GDA_Command_execute(impl_POA_GDA_Command * servant,
-			 CORBA_char * cmd,
-			 CORBA_long * affected,
-			 GDA_CmdParameterSeq * params,
-			 CORBA_long options,
-			 CORBA_Environment * ev)
+static CORBA_char *
+impl_GDA_Command__get_text(impl_POA_GDA_Command * servant,
+			   CORBA_Environment * ev)
 {
-  GDA_Recordset              new_recset;
-  Gda_ODBC_Recordset*        recset;
-  
-  *affected = 0;
-  if (servant->cmd->cmd) {
-    fprintf(stderr,"WARNING WARNING: command string still existing\n");
+   CORBA_char *retval;
+   if (!servant->cmd->cmd)
+     retval = 0;
+   else
+     retval = CORBA_string_dup(servant->cmd->cmd);
+   return retval;
+}
+
+static void
+impl_GDA_Command__set_text(impl_POA_GDA_Command * servant,
+			   CORBA_char * value,
+			   CORBA_Environment * ev)
+{
+  g_print("%s called, value = '%s'\n", __PRETTY_FUNCTION__, value);
+  if (servant->cmd->cmd)
     g_free(servant->cmd->cmd);
-  }
-  servant->cmd->cmd = g_strdup(cmd);
-  recset = gda_odbc_cmd_execute(servant->cmd, params, (gulong*)affected, 0);
-  g_warning("gda_odbc_cmd_execute returns NULL recordset");
-  if (!recset && servant->cmd->cnc->errors)
+  servant->cmd->cmd = g_strdup(value);
+}
+
+static CORBA_unsigned_long
+impl_GDA_Command__get_type(impl_POA_GDA_Command * servant,
+			   CORBA_Environment * ev)
+{
+  CORBA_unsigned_long retval;
+  retval = servant->cmd->type;
+  return retval;
+}
+
+static void
+impl_GDA_Command__set_type(impl_POA_GDA_Command * servant,
+			   CORBA_unsigned_long value,
+			   CORBA_Environment * ev)
+{
+  servant->cmd->type = value;
+}
+static GDA_Recordset
+impl_GDA_Command_open(impl_POA_GDA_Command * servant,
+		      GDA_CmdParameterSeq * params,
+		      GDA_CursorType ct,
+		      GDA_LockType lt,
+		      CORBA_unsigned_long * affected,
+		      CORBA_Environment * ev)
+{
+  
+  Gda_ODBC_Command*        gda_cmd;
+  Gda_ODBC_Connection*     gda_connection;
+  Gda_ODBC_Recordset*      recset;
+  GDA_Recordset            new_recset;
+  
+  g_print("%s called\n", __PRETTY_FUNCTION__);
+  gda_cmd = servant->cmd;
+  *affected = 0;
+  CORBA_exception_init(ev);
+  gda_connection = servant->cmd->cnc;
+  recset  = gda_odbc_cmd_execute(gda_cmd, params, (gulong*)affected, 0);
+  if (!recset && gda_connection->errors)
     {
       GDA_DriverError*     exception = GDA_DriverError__alloc();
-      Gda_ODBC_Connection* cnc = servant->cmd->cnc;
       
-      exception->errors._length = g_list_length(cnc->errors);
-      exception->errors._buffer = make_errorbuffer(cnc);
-      exception->realcommand = CORBA_string_dup((gchar*)cmd);
+      exception->errors._length = g_list_length(gda_connection->errors);
+      exception->errors._buffer = make_errorbuffer(gda_connection);
+      exception->realcommand = CORBA_string_dup(gda_cmd->cmd);
       CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
     }
   if (!recset)
     return CORBA_OBJECT_NIL;
   new_recset = impl_GDA_Recordset__create(servant->poa, recset, ev);
-  if (Exception(ev))
-    {
-      g_error("%s: Cannot create new recordset object", __PRETTY_FUNCTION__);
-    }
   return new_recset;
 }
 
+
+
 static GDA_Connection 
 impl_GDA_Connection__create(PortableServer_POA poa, CORBA_Environment * ev)
 {
@@ -1164,19 +1191,6 @@ impl_GDA_Connection_close(impl_POA_GDA_Connection * servant,
   return 0;
 }
 
-CORBA_long
-impl_GDA_Connection_execute(impl_POA_GDA_Connection * servant,
-			    CORBA_char * command,
-			    CORBA_long rowcount,
-			    CORBA_long stmt_type,
-			    CORBA_Environment * ev)
-{
-   CORBA_long retval = -1;
-
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
-   return retval;
-}
-
 CORBA_long
 impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
 			 CORBA_char * dsn,
@@ -1186,7 +1200,11 @@ impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
 {
   if (gda_odbc_connection_open(servant->cnc, dsn, user, passwd) < 0)
     {
-      g_warning("gda_Connection_open Error left");
+      GDA_DriverError* exception = GDA_DriverError__alloc();
+      exception->errors._length = g_list_length(servant->cnc->errors);
+      exception->errors._buffer = make_errorbuffer(servant->cnc);
+      exception->realcommand = CORBA_string_dup("Connection Open");
+      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
       return -1;
     }
   return 0;
@@ -1236,10 +1254,17 @@ impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
 				    CORBA_Environment * ev)
 {
    GDA_Recordset retval = 0;
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   Gda_ODBC_Recordset* rs = gda_odbc_recset_new();
+
+   g_print("GDA_Connection_createRecordset called\n");
+   rs->cnc = servant->cnc;
+   retval = impl_GDA_Recordset__create(servant->poa, rs, ev);
+   if (Exception(ev))
+     return CORBA_OBJECT_NIL;
    return retval;
 }
 
+
 GDA_Connection_DSNlist *
 impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
 				CORBA_Environment * ev)
@@ -1250,38 +1275,6 @@ impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
    return retval;
 }
 
-GDA_Recordset
-impl_GDA_Connection_openRecordset(impl_POA_GDA_Connection * servant,
-				  CORBA_char * command,
-				  GDA_LockType lock_type,
-				  GDA_CursorType cursor_type,
-				  CORBA_unsigned_long options,
-				  CORBA_Environment * ev)
-{
-  GDA_Recordset            new_recset;
-  Gda_ODBC_Recordset*      recset;
-  Gda_ODBC_Command*        cmd;
-  gulong                   affected;
-  
-  cmd = gda_odbc_cmd_new();
-  cmd->cmd = g_strdup(command);
-  cmd->cnc  = servant->cnc;
-  recset = gda_odbc_cmd_execute(cmd, 0, &affected, 0);
-  if (!recset)
-    {
-      GDA_DriverError*     exception = GDA_DriverError__alloc();
-      Gda_ODBC_Connection* odbc_cnc = servant->cnc;
-      
-      exception->errors._length = g_list_length(odbc_cnc->errors);
-      exception->errors._buffer = make_errorbuffer(odbc_cnc);
-      exception->realcommand = CORBA_string_dup((CORBA_char*)command);
-      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
-      return CORBA_OBJECT_NIL;
-    }
-  new_recset = impl_GDA_Recordset__create(servant->poa, recset, ev);
-  return new_recset;
-}
-
 CORBA_long
 impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
 				 CORBA_char * filename,
@@ -1579,19 +1572,6 @@ impl_GDA_Recordset_moveLast(impl_POA_GDA_Recordset * servant,
 }
 
 
-CORBA_long
-impl_GDA_Recordset_open(impl_POA_GDA_Recordset * servant,
-			CORBA_char * cmd,
-			GDA_Connection cnc,
-			GDA_CursorType ct,
-			GDA_LockType lt,
-			CORBA_Environment * ev)
-{
-   CORBA_long retval = -1;
-
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
-   return retval;
-}
 
 CORBA_long
 impl_GDA_Recordset_reQuery(impl_POA_GDA_Recordset * servant,
@@ -1662,9 +1642,8 @@ impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
   GList*                 tmp_rows;
   gint                   rowlength;
 
-  fprintf(stderr,"impl_GDA_Recordset_fetch: enter\n");
   if (!servant->rs)
-    return CORBA_OBJECT_NIL;
+    return GDA_Recordset_Chunk__alloc();	/* return empty chunk on error */
 
   tmp_rows = 0;
   rowidx = 0;
@@ -1735,19 +1714,18 @@ impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
 	  field = &row->_buffer[++colidx];
 	}
       rowidx++;
-  }while (rowidx < count);
-  fprintf(stderr,"After fetch loop\n");
+  } while (rowidx < count);
   if (rc < 0)
     {
       GDA_DriverError*     exception = GDA_DriverError__alloc();
-      Gda_ODBC_Connection* cnc = rs->cmd->cnc;
+      Gda_ODBC_Connection* cnc = rs->cnc;
       
       fprintf(stderr,"Ooops an error ocurred while fetching data\n");
       exception->errors._length = g_list_length(cnc->errors);
       exception->errors._buffer = make_errorbuffer(cnc);
       exception->realcommand = CORBA_string_dup("Fetch");
       CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
-      return CORBA_OBJECT_NIL;
+      return GDA_Recordset_Chunk__alloc();
     }
   chunk = GDA_Recordset_Chunk__alloc();
   if (rowidx)
@@ -1833,7 +1811,7 @@ impl_GDA_ConnectionFactory__create(PortableServer_POA poa, CORBA_Environment * e
 }
 
 /* You shouldn't call this routine directly without first deactivating the servant... */
-void
+static void
 impl_GDA_ConnectionFactory__destroy(impl_POA_GDA_ConnectionFactory * servant, CORBA_Environment * ev)
 {
 
