@@ -42,21 +42,28 @@ typedef struct
 
 
 
-CORBA_long    gda_Connection_close        (PortableServer_Servant _obj, CORBA_Environment* env);
-CORBA_long    gda_Connection_open         (PortableServer_Servant _obj,
+CORBA_long     gda_Connection_close        (PortableServer_Servant _obj, CORBA_Environment* env);
+CORBA_long     gda_Connection_open         (PortableServer_Servant _obj,
 					   const CORBA_char* dsn,
 					   const CORBA_char* user,
 					   const CORBA_char* passwd,
 					   CORBA_Environment* env);
-GDA_Recordset gda_Connection_openSchema   (PortableServer_Servant _obj,
+GDA_Recordset  gda_Connection_openSchema   (PortableServer_Servant _obj,
 					   const GDA_Connection_QType t,
 					   const GDA_Connection_ConstraintSeq* constraints,
 					   CORBA_Environment * ev);
-GDA_Command   gda_Connection_createCommand(PortableServer_Servant _obj, CORBA_Environment* ev);
+GDA_Command    gda_Connection_createCommand(PortableServer_Servant _obj, CORBA_Environment* ev);
 GDA_Connection_DSNlist* gda_Connection_listSources(PortableServer_Servant _obj, CORBA_Environment* ev);
-CORBA_long    GDA_odbc_begin_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
-CORBA_long    GDA_odbc_commit_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
-CORBA_long    GDA_odbc_rollback_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
+CORBA_long     GDA_odbc_begin_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
+CORBA_long     GDA_odbc_commit_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
+CORBA_long     GDA_odbc_rollback_transaction(PortableServer_Servant _obj, CORBA_Environment* ev);
+GDA_ErrorSeq*  GDA_odbc_connection_get_errors(PortableServer_Servant _obj, CORBA_Environment* ev);
+GDA_Recordset  GDA_odbc_connection_openRecordset(PortableServer_Servant _obj, const CORBA_char* txt,
+						 const GDA_LockType locktype, const GDA_CursorType cursor_type,
+						 const CORBA_unsigned_long otpions, CORBA_Environment* ev);
+CORBA_long     GDA_odbc_connection_start_logging(PortableServer_Servant _obj,  const CORBA_char*  filename,
+						 CORBA_Environment* ev);
+CORBA_long     GDA_odbc_connection_stop_logging(PortableServer_Servant _obj, CORBA_Environment* ev);
 
 POA_GDA_Connection__epv poa_gda_connection__epv = {
   NULL,				/* private */
@@ -75,8 +82,7 @@ POA_GDA_Connection__epv poa_gda_connection__epv = {
   (gpointer)13,				/* get_isOpen */
   (gpointer)14,                         /* set_isOpen */
   (gpointer)15,				/* get_version */
-  (gpointer)16,				/* get_errors */
-  (gpointer)17,				/* set_errors */
+  GDA_odbc_connection_get_errors,	/* get_errors */
   GDA_odbc_begin_transaction,		/* begin Transaction */
   GDA_odbc_commit_transaction,		/* commit Transaction */
   GDA_odbc_rollback_transaction,        /* rollback Transaction */
@@ -85,7 +91,10 @@ POA_GDA_Connection__epv poa_gda_connection__epv = {
   gda_Connection_open,		        /* open */
   gda_Connection_openSchema, 	        /* openSchema */
   gda_Connection_createCommand,         /* createCommand */
-  gda_Connection_listSources            /* listSources */
+  gda_Connection_listSources,           /* listSources */
+  GDA_odbc_connection_openRecordset,     /* openRecordset */
+  GDA_odbc_connection_start_logging,
+  GDA_odbc_connection_stop_logging
 };
 
 
