@@ -33,20 +33,16 @@ gda_postgres_cmd_new (void)
 void
 gda_postgres_cmd_free (Gda_POSTGRES_Command *cmd)
 {
-  if (cmd->cmd != NULL)
+  if (cmd->cmd != 0)
     g_free((gpointer) cmd->cmd);
-  if (cmd->pq_data != NULL)
-    {
-      PQclear(cmd->pq_data);
-      cmd->pq_data = NULL;
-    }
-  cmd->cmd = NULL;
+  cmd->cmd = 0;
   /* free rowsets? */
   g_free((gpointer) cmd);
 }
 
 Gda_POSTGRES_Connection *
-gda_postgres_cmd_set_connection (Gda_POSTGRES_Command *cmd, Gda_POSTGRES_Connection *cnc)
+gda_postgres_cmd_set_connection (Gda_POSTGRES_Command *cmd,
+                                 Gda_POSTGRES_Connection *cnc)
 {
   Gda_POSTGRES_Connection *rc = cmd->cnc;
   cmd->cnc = cnc;
@@ -59,17 +55,17 @@ init_recset_fields (Gda_POSTGRES_Recordset *recset)
   register gint cnt;
   /* check parameters */
   g_return_val_if_fail(recset != NULL, NULL);
-  for (cnt = 0; cnt < PQnfields(recset->cmd->pq_data); cnt++)
+  for (cnt = 0; cnt < PQnfields(recset->pq_data); cnt++)
     {
       Gda_POSTGRES_Field *f = gda_postgres_field_new();
-      f->name = g_strdup(PQfname(recset->cmd->pq_data, cnt));
+      f->name = g_strdup(PQfname(recset->pq_data, cnt));
       //f->sql_type = ;
       //f->c_type = ;
       //f->nullable = ;
       f->value = NULL;
       f->precission = 0;
       f->num_scale = 0;
-      f->defined_length = f->actual_length = PQfsize(recset->cmd->pq_data, cnt);
+      f->defined_length = f->actual_length = PQfsize(recset->pq_data, cnt);
       f->malloced = 0;
       /* add field definition to list */
       recset->fields = g_list_append(recset->fields, f);
@@ -78,24 +74,26 @@ init_recset_fields (Gda_POSTGRES_Recordset *recset)
 }
 
 Gda_POSTGRES_Recordset *
-gda_postgres_cmd_execute (Gda_POSTGRES_Command *cmd, Gda_POSTGRES_Error *error,
-			  const GDA_CmdParameterSeq *params, gulong options)
+gda_postgres_cmd_execute (Gda_POSTGRES_Command *cmd,
+                          Gda_POSTGRES_Error *error,
+                          const GDA_CmdParameterSeq *params,
+                          gulong options)
 {
   Gda_POSTGRES_Recordset *recset;
+
   /* create recordset object */
   recset = gda_postgres_recset_new();
   fprintf(stderr, "gda_postgres_cmd_execute(): enter\n");
+  
   /* FIXME: parameters stuff, ask Michael */
   /* execute query */
-  recset->cmd = cmd;
-  cmd->pq_data = PQexec(cmd->cnc->pq_conn, cmd->cmd);
-  if (cmd->pq_data != NULL)
+  recset->pq_data = PQexec(cmd->cnc->pq_conn, cmd->cmd);
+  if (recset->pq_data != 0)
     {
-      switch (PQresultStatus(cmd->pq_data))
+      switch (PQresultStatus(recset->pq_data))
 	{
 	case PGRES_EMPTY_QUERY :
 	case PGRES_TUPLES_OK :
-	  /* FIXME: what has to be done here? */
 	  recset->at_end = FALSE;
 	  recset->at_begin = TRUE;
 	  recset->pos = 0;
@@ -104,15 +102,15 @@ gda_postgres_cmd_execute (Gda_POSTGRES_Command *cmd, Gda_POSTGRES_Error *error,
 	case PGRES_COPY_OUT :
 	case PGRES_COPY_IN :
 	  recset->at_begin = recset->at_end = TRUE;
-	  recset->fields = NULL;
+	  recset->fields = 0;
 	  recset->pos = 0;
 	  return (recset);
 	default :
 	  gda_postgres_recset_free(recset);
 	  gda_postgres_error_make(error, recset, cmd->cnc, "PQexec");
-	  return (NULL);
+	  return (0);
 	}
     }
   else gda_postgres_error_make(error, recset, cmd->cnc, "PQexec");
-  return (NULL);
+  return (0);
 }
