@@ -25,8 +25,7 @@
 #include "gda-postgres-error.h"
 #include "gda-postgres-recset.h"
 
-typedef Gda_POSTGRES_Recordset* (*schema_ops_fn)(Gda_POSTGRES_Recordset *,
-						 Gda_POSTGRES_Error *,
+typedef Gda_POSTGRES_Recordset* (*schema_ops_fn)(Gda_POSTGRES_Error *,
 						 Gda_POSTGRES_Connection *,
 						 GDA_Connection_Constraint *,
 						 gint );
@@ -40,8 +39,8 @@ schema_ops_fn schema_ops[GDA_Connection_GDNC_SCHEMA_LAST] =
 static gint
 execute_command (Gda_POSTGRES_Connection *cnc, gchar *cmd)
 {
-  g_return_val_if_fail(cnc != NULL, -1);
-  g_return_val_if_fail(cmd != NULL, -1);
+  g_return_val_if_fail(cnc != 0, -1);
+  g_return_val_if_fail(cmd != 0, -1);
   if (PQstatus(cnc->pq_conn) == CONNECTION_OK)
     {
       PGresult *rc = PQexec(cnc->pq_conn, cmd);
@@ -59,27 +58,30 @@ execute_command (Gda_POSTGRES_Connection *cnc, gchar *cmd)
 	case PGRES_FATAL_ERROR :
 	  break;
         }
-      gda_postgres_error_make(NULL, NULL, cnc, "PQexec");
-      if (rc != NULL)
+      gda_postgres_error_make(0, 0, cnc, "PQexec");
+      if (rc != 0)
         PQclear(rc);
     }
   return (-1);
 }
 
 static Gda_POSTGRES_Recordset *
-schema_columns (Gda_POSTGRES_Recordset *recset, Gda_POSTGRES_Error *error,
+schema_columns (Gda_POSTGRES_Error *error,
                 Gda_POSTGRES_Connection *cnc, GDA_Connection_Constraint *constraint,
                 gint length)
 {
   Gda_POSTGRES_Command *cmd;
+  Gda_POSTGRES_Recordset *recset = 0;
   GDA_Connection_Constraint *ptr;
   gchar *table_qualifier, *table_owner, *table_name, *column_name;
+  
   /* create the command object */
   cmd = gda_postgres_cmd_new();
   cmd->cnc = cnc;
+  
   /* get table name */
   ptr = constraint;
-  while(length)
+  while (length)
     {
       switch (ptr->ctype)
 	{
@@ -107,63 +109,85 @@ schema_columns (Gda_POSTGRES_Recordset *recset, Gda_POSTGRES_Error *error,
       length--;
       ptr++;
     }
-  /* FIXME: missing fields: COLUMN_NAME, DATA_TYPE, TYPE_NAME, DATA_PRECISION,
+  /* FIXME: missing fields: DATA_TYPE, TYPE_NAME, DATA_PRECISION,
                             LENGTH, NUMERIC_SCALE, NUMERIC_RADIX, NULLABLE, REMARKS */
-  cmd->cmd = g_strdup_printf("SELECT a.relname, c.usename, a.relname "
-                             " FROM pg_attribute d, pg_user c, pg_database b, pg_class a "
-                             " WHERE a.relowner = b.datdba AND c.usesysid = b.datdba "
-                             " AND a.oid = d.attrelid "
-                             " AND b.datname = %s "
-                             " AND a.relname = %s ",
+  cmd->cmd = g_strdup_printf("SELECT b.datname AS \"Catalog\","
+                             "       c.usename AS \"Schema\","
+                             "       a.tablename AS \"Table\", "
+                             "       e.attname AS \"Column\" "
+                             " FROM pg_attribute e, "
+                             "      pg_class d, "
+                             "      pg_user c, "
+                             "      pg_database b, "
+                             "      pg_tables a "
+                             " WHERE a.tableowner = c.usename "
+                             " AND c.usesysid = b.datdba "
+                             " AND d.oid = e.attrelid "
+                             " AND d.relname = a.tablename "
+                             " AND b.datname = '%s' "
+                             " AND a.tablename = '%s' "
+                             " ORDER BY e.attname",
                              PQdb(cnc->pq_conn), table_name);
-  /* not finished */
+
+  /* execute the command */
+  recset = gda_postgres_cmd_execute(cmd, error, 0, 0);
   return (recset);
 }
 
 static Gda_POSTGRES_Recordset *
-schema_procedures (Gda_POSTGRES_Recordset *recset, Gda_POSTGRES_Error *error,
+schema_procedures (Gda_POSTGRES_Error *error,
                    Gda_POSTGRES_Connection *cnc, GDA_Connection_Constraint *constraint,
                    gint length)
 {
+  Gda_POSTGRES_Recordset *recset = 0;
   return (recset);
 }
 
 static Gda_POSTGRES_Recordset *
-schema_tables (Gda_POSTGRES_Recordset *recset, Gda_POSTGRES_Error *error,
+schema_tables (Gda_POSTGRES_Error *error,
 	       Gda_POSTGRES_Connection *cnc, GDA_Connection_Constraint *constraint,
 	       gint length)
 {
   gchar *query;
   Gda_POSTGRES_Command *cmd;
+  Gda_POSTGRES_Recordset *recset = 0;
   /* build the command object */
   cmd = gda_postgres_cmd_new();
   cmd->cnc = cnc;
   /* build the query */
-  query = g_strdup_printf("SELECT a.relname, c.usename, a.relname, 'TABLE', null "
-                          " FROM pg_user c, pg_database b, pg_class a "
-                          " WHERE b.datname = '%s' "
-                          " AND a.relowner = b.datdba AND c.usesysid = b.datdba"
-			  " ORDER BY a.relname",
-                          PQdb(cnc->pq_conn));
+  query = g_strdup_printf("SELECT a.tablename, a.tableowner, a.tablename, 'TABLE', null "
+                          " FROM pg_database c, pg_class b, pg_tables a "
+                          " WHERE a.tableowner = '%s' "
+                          " AND a.tablename = b.relname "
+                          " AND b.relowner = c.datdba "
+                          " AND c.datname = '%s' ",
+                          PQuser(cnc->pq_conn), PQdb(cnc->pq_conn));
   cmd->cmd = query;
-  recset->cmd = cmd;
+
   /* execute the command */
-  cmd->pq_data = PQexec(cnc->pq_conn, cmd->cmd);
-  switch (PQresultStatus(cmd->pq_data))
-    {
-    case PGRES_EMPTY_QUERY :
-    case PGRES_TUPLES_OK :
-    case PGRES_COMMAND_OK :
-    case PGRES_COPY_OUT :
-    case PGRES_COPY_IN :
-      return (recset);
-    case PGRES_BAD_RESPONSE :
-    case PGRES_NONFATAL_ERROR :
-    case PGRES_FATAL_ERROR :
-      break;
-    }
-  gda_postgres_error_make(error, recset, cnc, "schema_tables");
-  return (0);
+  recset = gda_postgres_cmd_execute(cmd, error, 0, 0);
+  return (recset);
+}
+
+static Gda_POSTGRES_Recordset *
+schema_views (Gda_POSTGRES_Error *error,
+	       Gda_POSTGRES_Connection *cnc, GDA_Connection_Constraint *constraint,
+	       gint length)
+{
+  gchar *query;
+  Gda_POSTGRES_Command *cmd;
+  Gda_POSTGRES_Recordset *recset = 0;
+  /* build the command object */
+  cmd = gda_postgres_cmd_new();
+  cmd->cnc = cnc;
+  
+  /* build the query */
+  query = g_strdup_printf("SELECT * FROM pg_views");
+  cmd->cmd = query;
+
+  /* execute the command */
+  recset = gda_postgres_cmd_execute(cmd, error, 0, 0);
+  return (recset);
 }
 
 static void
@@ -172,6 +196,7 @@ initialize_schema_ops (void)
   schema_ops[GDCN_SCHEMA_TABLES] = schema_tables;
   schema_ops[GDCN_SCHEMA_COLS] = schema_columns;
   schema_ops[GDCN_SCHEMA_PROCS] = schema_procedures;
+  schema_ops[GDCN_SCHEMA_VIEWS] = schema_views;
 }
 
 /* create new connection object */
@@ -185,14 +210,17 @@ gda_postgres_connection_new (void)
       initialize_schema_ops();
       initialized = 1;
     }
-  /* initialize everything to NULL to avoid core dumped */
-  c->pq_host = NULL;
-  c->pq_port = NULL;
-  c->pq_options = NULL;
-  c->pq_tty = NULL;
-  c->pq_db = NULL;
-  c->pq_login = NULL;
-  c->pq_pwd = NULL;
+
+  /* initialize everything to 0 to avoid core dumped */
+  c->pq_host = 0;
+  c->pq_port = 0;
+  c->pq_options = 0;
+  c->pq_tty = 0;
+  c->pq_db = 0;
+  c->pq_login = 0;
+  c->pq_pwd = 0;
+  c->pq_conn = 0;
+  c->errors = 0;
   return (c);
 }
 
@@ -200,15 +228,15 @@ gda_postgres_connection_new (void)
 void
 gda_postgres_connection_free (Gda_POSTGRES_Connection *c)
 {
-  g_return_if_fail(c != NULL);
+  g_return_if_fail(c != 0);
   gda_postgres_connection_close(c);
-  if (c->pq_host != NULL) g_free((gpointer) c->pq_host);
-  if (c->pq_port != NULL) g_free((gpointer) c->pq_port);
-  if (c->pq_options != NULL) g_free((gpointer) c->pq_options);
-  if (c->pq_tty != NULL) g_free((gpointer) c->pq_tty);
-  if (c->pq_db != NULL) g_free((gpointer) c->pq_db);
-  if (c->pq_login != NULL) g_free((gpointer) c->pq_login);
-  if (c->pq_pwd != NULL) g_free((gpointer) c->pq_pwd);
+  if (c->pq_host != 0) g_free((gpointer) c->pq_host);
+  if (c->pq_port != 0) g_free((gpointer) c->pq_port);
+  if (c->pq_options != 0) g_free((gpointer) c->pq_options);
+  if (c->pq_tty != 0) g_free((gpointer) c->pq_tty);
+  if (c->pq_db != 0) g_free((gpointer) c->pq_db);
+  if (c->pq_login != 0) g_free((gpointer) c->pq_login);
+  if (c->pq_pwd != 0) g_free((gpointer) c->pq_pwd);
   g_free((gpointer) c);
 }
 
@@ -257,16 +285,18 @@ gda_postgres_connection_open (Gda_POSTGRES_Connection *cnc, const gchar *dsn,
       if (ptr_s)
 	ptr_s++;
     }
-  if (cnc->pq_login == NULL) 
-    cnc->pq_login = user != NULL ? g_strdup(user) : NULL;
-  if (cnc->pq_pwd == NULL)
-    cnc->pq_pwd = password != NULL ? g_strdup(password) : NULL;
+  if (!cnc->pq_login) 
+    cnc->pq_login = user != 0 ? g_strdup(user) : 0;
+  if (!cnc->pq_pwd)
+    cnc->pq_pwd = password != 0 ? g_strdup(password) : 0;
   /* actually establish the connection */
   cnc->pq_conn = PQsetdbLogin(cnc->pq_host, cnc->pq_port, cnc->pq_options,
 			      cnc->pq_tty, cnc->pq_db, cnc->pq_login, cnc->pq_pwd);
   if (PQstatus(cnc->pq_conn) != CONNECTION_OK)
     {
-      fprintf(stderr, "PostgreSQL error: %s\n", PQerrorMessage(cnc->pq_conn));
+      Gda_POSTGRES_Error* error;
+      error = gda_postgres_error_new();
+      gda_postgres_error_make(error, 0, cnc, "gda_postgres_connection_open");
       return (-1);
     }
   return (0);
@@ -280,15 +310,13 @@ gda_postgres_connection_open_schema(Gda_POSTGRES_Connection *cnc,
 				    GDA_Connection_Constraint *constraints,
 				    gint length)
 {
-  Gda_POSTGRES_Recordset* rc = gda_postgres_recset_new();
   schema_ops_fn fn = schema_ops[(gint) t];
 
   if (fn)
-    return fn(rc, e, cnc, constraints, length);
+    return fn(e, cnc, constraints, length);
   else
     g_log("GDA POSTGRES", G_LOG_LEVEL_INFO,
 	  "schema_open_schema: Unhandled SCHEMA_QTYPE %d\n", (gint) t);
-  gda_postgres_recset_free(rc);
   return 0;
 }
 
@@ -300,10 +328,10 @@ gda_postgres_connection_close (Gda_POSTGRES_Connection *c)
   if (PQstatus(c->pq_conn) == CONNECTION_OK)
     {
       PQfinish(c->pq_conn);
-      c->pq_conn = NULL;
+      c->pq_conn = 0;
       return (0);
     }
-  gda_postgres_error_make(NULL, NULL, c, "gda_postgres_connection_close");
+  gda_postgres_error_make(0, 0, c, "gda_postgres_connection_close");
   return (-1);
 }
 
@@ -334,11 +362,13 @@ gda_postgres_connection_start_logging (Gda_POSTGRES_Connection *cnc, gchar *file
 {
   FILE *f;
   gchar *str;
+  
   /* check parameters */
-  g_return_val_if_fail(cnc != NULL, -1);
+  g_return_val_if_fail(cnc != 0, -1);
   if (PQstatus(cnc->pq_conn) != CONNECTION_OK)
     return (-1);
-  /* open given file or default one if NULL */
+    
+  /* open given file or default one if 0 */
   str = filename ? filename : g_strdup_printf("%s/gda.log", g_get_home_dir());
   if (!(f = fopen(str, "w")))
     {
@@ -357,9 +387,16 @@ gint
 gda_postgres_connection_stop_logging (Gda_POSTGRES_Connection *cnc)
 {
   /* check parameters */
-  g_return_val_if_fail(cnc != NULL, -1);
+  g_return_val_if_fail(cnc != 0, -1);
   if (PQstatus(cnc->pq_conn) != CONNECTION_OK)
     return (-1);
   PQuntrace(cnc->pq_conn);
   return (0);
 }
+
+gchar *
+gda_postgres_connection_create_recset (Gda_POSTGRES_Connection *cnc,
+                                       GDA_RowAttributes *columns)
+{
+  return (0);
+}
\ No newline at end of file
