@@ -34,6 +34,8 @@ Gda_POSTGRES_Recordset  *gda_postgres_connection_open_schema (Gda_POSTGRES_Conne
 							      GDA_Connection_QType t,
 							      GDA_Connection_Constraint *constraints,
 							      gint length);
+gchar*                   gda_postgres_connection_create_recset (Gda_POSTGRES_Connection* cnc,
+                                                                GDA_RowAttributes* columns);
 
 /*
  * Native library logging
