@@ -41,12 +41,14 @@ typedef struct _Gda_POSTGRES_Command
 {
   struct _Gda_POSTGRES_Connection* cnc;
   gchar*                           cmd;
-  PGresult*                        pq_data;
+  gulong                           type;
 } Gda_POSTGRES_Command;
 
 typedef struct _Gda_POSTGRES_Recordset
 {
-  struct _Gda_POSTGRES_Command *cmd;
+  /*  struct _Gda_POSTGRES_Command *cmd;*/
+  PGresult*                     pq_data;
+  Gda_POSTGRES_Connection*      cnc;
   gint                          at_begin;
   gint                          at_end;
   gulong                        pos;
