@@ -42,7 +42,7 @@ main(int argc, char* argv[])
   sigaction(SIGPIPE, &sa, 0);
   
   logstream = fopen("/tmp/gda-postgres-srv.log", "a");
-  g_log_set_handler("GDA ODBC", 0xffffffff, g_log_2_file, logstream);
+  g_log_set_handler("GDA POSTGRES", 0xffffffff, g_log_2_file, logstream);
 
 #if 0
   while (wait_for_debugger)
