@@ -131,11 +131,7 @@ main(int argc, char* argv[])
     } while (!gda_recordset_eof(rs));
   fprintf(stderr,"EOF in recordset, closing connection\n");
   getchar();
-  if (gda_connection_close(cnc) < 0)
-    {
-      fprintf(stderr,"gda_connection_close failed\n");
-      return -1;
-    }
+  gda_connection_close(cnc);
   return 0;
 }
 
